
/*
extended functions to jpair.c in the Jabber project (www.jabber.org)
these routines compare strings with strcasecmp, instead of strcmp

also:
 free_ejpair_val() to free the jpair but the keys
 ejpair_replace() to replace a value in a jpair

disq <disq@iname.com>
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "common.h"

jpair *ejpair_get(jpair *p, char *key)
{
	while(p != NULL)
	{
		if(p->key != NULL && key != NULL && strcasecmp(p->key,key) == 0)
			return p;
		p = p->next;
	}
	return NULL;
}

char *ejpair_getval(jpair *p, char *key)
{
	while(p != NULL)
	{
		if(p->key != NULL && key != NULL && strcasecmp(p->key,key) == 0)
			return p->val;
		p = p->next;
	}
	return NULL;
}

char *ejpair_getvalnew(jpair *p, char *key)
{
	char *val;

	val = ejpair_getval(p, key);
	if(val != NULL)
		return strdup(val);
	return NULL;
}

int ejpair_getflag(jpair *p, char *key)
{
	while(p != NULL)
	{
		if(p->key != NULL && key != NULL && strcasecmp(p->key,key) == 0)
			return p->flag;
		p = p->next;
	}
	return 0;
}

void free_ejpair_val(jpair *p)
{
	jpair *next;

	next = p;

	while(next != NULL)
	{
		p = next;
		next = p->next;
		free(p->val);
		free(p);
	}
}


jpair *ejpair_replace(jpair *p, char *key, char *val)
{
jpair *j;
jpair *r;
char *oldval;

r=p;
j=ejpair_get(p,key);
if (j==NULL)
	{
	r=jpair_add(p,strdup(key),val);
	return(r);
	}
oldval=j->val;
if (strcasecmp(oldval,val)==0) return(r);
if (oldval!=NULL) free(oldval);
j->val=val;
return(r);
}
