#include <gtk/gtk.h>
#include "common-cab.h"
#include "common.h"
#include "mpair.h"
#include "ejpair.h"
#include "warper.h"
#include "events.h"
#include "main.h"
#include "gui.h"
#include "readwin.h"
#include "rcvwin.h"
#include "prefswin.h"
#include "menu.h"
#include "chatbymsg.h"


void read_sock(gpointer	data, gint source, GdkInputCondition condition)
{
char *buf;
ssize_t sz;

buf=g_malloc(4096);
sz=read(source,buf,/*sizeof(buf)-1*/4095);
  if (sz>0)
	{
	buf[sz]=0;
	printf("\nGOT: *%s*(%d)",buf,sz);
	fflush(stdout);
	events_got_stream(buf);
	}
  else
	{
	fprintf(stderr,"\nServer closed connection");
	fflush(stderr);
	gdk_input_remove(tag);
	sock=-1;
	events_free();
	events_init();
	gtk_widget_set_sensitive(gtk_item_factory_get_widget(item_factory, "<main>/User/Login"),TRUE);
	gtk_widget_set_sensitive(gtk_item_factory_get_widget(item_factory, "<main>/User/Logout"),FALSE);
	}

g_free(buf);
}

void login(char *user, char *pass, char *nick)
{
send_login(user, pass, nick);
send_status(ourstatus, ourtext->str, ourpri, ouricon);
send_roster_get(NULL);
}

void tag_unknown(xpt *x)
{
fprintf(stderr,"\nUNKNOWN TAG RECEIVED: %s",x->name);
}


void do_message(jpair *data)
{
char *fromid, *fromnick, *subj, *thread, *pri, *msg, *tim;
static time_t t;
userstruct *u;
msg_struct *m;

fromid=jpair_getval(data,"fromid");
fromnick=jpair_getval(data,"fromnick");
subj=jpair_getval(data,"subject");
thread=jpair_getval(data,"thread");
pri=jpair_getval(data,"priority");
msg=jpair_getval(data,"msg");

time(&t);
tim=ctime(&t);

add_new_message_to_rcvwin(fromid, fromnick, subj, thread, pri, msg, tim);

u=uname2struc(fromid);
if (u==NULL) /* FIXME: is this approach enough? */
	{
	groupstruct *r;

	r=groupname2struc("unknown");	
	if (r==NULL) r=add_group("unknown");
	u=add_user(r, fromid, S_OFFLINE);
	}
if (u!=NULL)
  {
  m=g_malloc(sizeof(msg_struct));
  m->fromid=g_strdup(fromid);
  m->fromnick=g_strdup(fromnick);
  m->subj=g_strdup(subj);
  m->thread=g_strdup(thread);
  m->pri=g_strdup(pri);
  m->msg=g_strdup(msg);
  m->date=g_strdup(tim);


	if ((u->chatbymsg!=NULL) /*&& (strcmp((chatbymsg_struct  *)(u->chatbymsg)->thread->str,m->thread)==0)*/)
		{
		chatbymsg_newmsg((chatbymsg_struct *)(u->chatbymsg), m);
		}
	else
		{
 	 	u->msgs=g_list_append(u->msgs, (gpointer)m);
	 	u->num_msgs++;
  	if (u->readwin!=NULL) readwin_add((readwin_struct *)(u->readwin));
				   					else add_icon_to_queue(u,IQ_MESSAGE);
		}
  }
else
  {
  puts("\nuser not in list, message not saved into buffer");
  }

puts("\n**New message received**");

free_ejpair_val(data);
}


void do_status(jpair *data)
{
char *fromnick, *fromid, *statustxt;
signed int pri=0;
userstruct *u;
imstruct *im;
StatusType type;
IconType icon;
char *s;

fromid=jpair_getvalnew(data,"fromid");
fromnick=jpair_getvalnew(data,"fromnick");
statustxt=jpair_getvalnew(data,"msg");

s=jpair_getvalnew(data,"type");
if ((s!=NULL) && (strcasecmp(s,"online")==0)) type=S_ONLINE;
					                     else type=S_OFFLINE;
g_free(s);

s=jpair_getvalnew(data,"icon");
if ((s!=NULL) && (strcasecmp(s,"away")==0)) icon=I_AWAY;
 				                       else icon=I_NORMAL;
g_free(s);

s=jpair_getvalnew(data,"priority");
if (s!=NULL) pri=atoi(s);
g_free(s);


u=uname2struc(fromid);
if ((u!=NULL) && (fromnick!=NULL))
  {
  im=nick2struc(u,fromnick);
  if (im==NULL) add_transport(u, fromnick, type, statustxt, icon, pri);
		   else edit_transport(im, fromnick, type, statustxt, icon, pri);
  }

if (fromid==NULL)
  {
  printf("\nStatus message with no FromID, ignored");
  fflush(stdout);
  }
if (fromnick==NULL)
  {
  printf("\nStatus message with no FromNick, ignored");
  fflush(stdout);
  }

g_free(fromid);
g_free(fromnick);
g_free(statustxt);

free_ejpair_val(data);
}


void do_roster(jpair *data)
{
char *gname, *uid;
groupstruct *r=NULL;
userstruct *u=NULL;

gname=jpair_getvalnew(data,"groupname");
uid=jpair_getvalnew(data,"userid");

u=uname2struc(uid);
if (u==NULL)
  {
  r=mpair_getval(grouphash, gname);
  if ((r==NULL) && (gname!=NULL))
		{
		r=add_group(gname);
		}
  if ((uid!=NULL) && (r!=NULL)) add_user(r, uid, S_OFFLINE);
	if (uid==NULL) send_roster_get(gname);
  }
else
  {
  fprintf(stderr,"\nThe user server sent already exist in our roster, ignored");
  fflush(stderr);
  }

g_free(gname);
g_free(uid);

free_ejpair_val(data);
}


void events_init()
{
	wraphash=mpair_add(NULL,g_strdup("message"),&do_message);
	wraphash=mpair_add(wraphash,g_strdup("status"),&do_status);
	wraphash=mpair_add(wraphash,g_strdup("roster"),&do_roster);
    xpp = xpt_packet_new();
}


void events_free()
{
  XML_ParserFree(xpp->parser);
  free_mpair(wraphash);
}


void events_got_stream(char *buf)
{
xpt *xml, *next;

  xpt_packet_data(xpp,buf,strlen(buf));
  xml = xpp->packets;
  next = xml;
  
  if ((gotjabbertag==FALSE) && (xpp->name!=NULL) && (strcasecmp(xpp->name,"jabber")==0))
	{
	server_ver=ejpair_getval(xpp->attributes,"version");
	server_prot=ejpair_getval(xpp->attributes,"protocol");
	printf("\nServer version: %s\nServer protocol: %s",server_ver,server_prot);
	fflush(stdout);
/*
	if (strcmp(server_prot,our_prot)!=0)
	  {
	  fprintf(stderr,"\nServer is not using protocol version %s, aborted",our_prot);
	  exit(2);
	  }
*/
	gotjabbertag=TRUE;
	}

  while(next != NULL)
    {
	xml = next;
	next = next->next;
    xml->next = NULL;

	route_xml(xml, wraphash, &tag_unknown);
    }

	xpp->packets = NULL;
	switch(xpp->flag)
	{
		case XPT_POOLFLAG_CLOSE:
			printf("Found end, closing!\n");
			break;
		case XPT_POOLFLAG_ERROR:
		      fprintf(stderr,
            		  "%s at line %d\n",
            		  XML_ErrorString(XML_GetErrorCode(xpp->parser)),
            		  XML_GetCurrentLineNumber(xpp->parser));
		break;
	}
}


void send_data(char *s)
{
ssize_t sz;
int l;

if (sock>0)
  {
  l=strlen(s);
  sz=write(sock,s,l);

  if (sz<l)
	{
	fprintf(stderr,"\nERROR: %d out of %d bytes written",sz,l); 
	fflush(stderr);
	}
  }

printf("\nSEND: %s",s);
if (sock<1) printf("(not connected)");
fflush(stdout);
}

void send_hdr()
{
char s[1024];
sprintf(s,"<?xml version=\"1.0\"?><jabber version=\"%s %s\" protocol=\"%s\">",PACKAGE,VERSION,our_prot);
send_data(s);
}

void send_login(char *user, char *pass, char *nick)
{
char *buf = NULL;

  buf=strgrow(buf,"<login><user>",1,0);
  buf=strgrow(buf,strescape(user),1,1);
  buf=strgrow(buf,"</user><pass>",1,0);
  buf=strgrow(buf,strescape(pass),1,1);
  buf=strgrow(buf,"</pass><nick>",1,0);
  buf=strgrow(buf,strescape(nick),1,1);
  buf=strgrow(buf,"</nick></login>",1,0);
  send_data(buf);
  g_free(buf);
}

void send_status(StatusType type, char *say, int pri, IconType icon)
{
char *buf = NULL;
char s[64];

  buf=strgrow(buf,"<status",1,0);
/*
  if (type!=NULL)
	{
*/
	buf=strgrow(buf," type='",1,0);
	if (type==S_ONLINE) buf=strgrow(buf,"online",1,0);
				   			 else buf=strgrow(buf,"offline",1,0);
	buf=strgrow(buf,"'>",1,0);
/*
	}
  else
	{
	buf=strgrow(buf,">",1,0);
	}
*/
	buf=strgrow(buf,"<say>",1,0);
  buf=strgrow(buf,strescape(say),1,1);
  buf=strgrow(buf,"</say>",1,0);
/*
  if (pri!=NULL)
	{
*/
	sprintf(s,"%d",pri);
	buf=strgrow(buf,"<priority>",1,0);
	buf=strgrow(buf,s,1,0);
	buf=strgrow(buf,"</priority>",1,0);
/*	}
*/
/*
  if (icon!=NULL)
	{
*/
	buf=strgrow(buf,"<icon>",1,0);
	if (icon==I_AWAY) buf=strgrow(buf,"away",1,0);
				 else buf=strgrow(buf,"normal",1,0);
	buf=strgrow(buf,"</icon>",1,0);
/*	}*/
  buf=strgrow(buf,"</status>",1,0);
  send_data(buf);
  g_free(buf);
}

void send_roster_add(char *group, char *uname)
{
char *buf = NULL;

  buf=strgrow(buf,"<roster><add group='",1,0);
  buf=strgrow(buf,strescape(group),1,1);
  buf=strgrow(buf,"'",1,0);
  if (uname==NULL)
	{
	buf=strgrow(buf,"/>",1,0);
	}
  else
	{
	buf=strgrow(buf,">",1,0);
	buf=strgrow(buf,strescape(uname),1,1);
	buf=strgrow(buf,"</add>",1,0);
	}
  buf=strgrow(buf,"</roster>",1,0);

  send_data(buf);
  g_free(buf);
}

void send_roster_del(char *group, char *uname)
{
char *buf = NULL;

  buf=strgrow(buf,"<roster><del group='",1,0);
  buf=strgrow(buf,strescape(group),1,1);
  buf=strgrow(buf,"'",1,0);
  if (uname==NULL)
	{
	buf=strgrow(buf,"/>",1,0);
	}
  else
	{
	buf=strgrow(buf,">",1,0);
	buf=strgrow(buf,strescape(uname),1,1);
	buf=strgrow(buf,"</del>",1,0);
	}
  buf=strgrow(buf,"</roster>",1,0);

  send_data(buf);
  g_free(buf);
}


void send_roster_get(char *group)
{
char *buf = NULL;

  buf=strgrow(buf,"<roster><get",1,0);
  if (group==NULL)
	{
	buf=strgrow(buf,"/>",1,0);
	}
  else
	{
	buf=strgrow(buf," group='",1,0);
	buf=strgrow(buf,strescape(group),1,1);
	buf=strgrow(buf,"'/>",1,0);
	}
  buf=strgrow(buf,"</roster>",1,0);
  send_data(buf);
  g_free(buf);
}


void connect_login(char *uid, char *pass, char *nick)
{
char *server=NULL, *user=NULL, *uiddup=NULL;
char *x=NULL;

cfg=ejpair_replace(cfg,"user",g_strdup(uid));
if (gettoggle("dontsavepassword")==FALSE) cfg=ejpair_replace(cfg,"pass",g_strdup(pass));
cfg=ejpair_replace(cfg,"nick",g_strdup(nick));

uid=g_strdup(uid);
pass=g_strdup(pass);
nick=g_strdup(nick);

x=index(uid,'@');
if ((x!=NULL) && ((uid!=NULL) && (uid[0]!='@')))
  {
  server=g_strdup(strstr(uid,"@"));
	
  sock = make_netsocket(5222,&server[1],1);
  if (sock<0)
		{
		fprintf(stderr,"\nCould not connect");
		fflush(stderr);
		}
  else
		{
		puts("connected!");
		tag=gdk_input_add(sock, GDK_INPUT_READ, (GdkInputFunction)&read_sock, NULL);

		gtk_widget_set_sensitive(gtk_item_factory_get_widget(item_factory, "<main>/User/Login"),FALSE);
		gtk_widget_set_sensitive(gtk_item_factory_get_widget(item_factory, "<main>/User/Logout"),TRUE);
		
		if (gettoggle("uniconifyconnect"))
			{
			gdk_window_show(win->window);
			gdk_window_raise(win->window);
			}

		send_hdr();
		uiddup=g_strdup(uid);
		user=g_strdup(strtok(uiddup,"@")); /* strip the ip address from the user id */

		ourserver=g_string_assign(ourserver, g_strdup(server));
		ouruser=g_string_assign(ouruser, g_strdup(user));
		ournick=g_string_assign(ournick, g_strdup(nick));

		login(user, pass, nick);
		}
  }
else
  {
  fprintf(stderr,"\nEnter userid in the format userid@server.com");
  fflush(stderr);
  }

g_free(user);
g_free(server);
g_free(uid);
g_free(uiddup);
g_free(pass);
g_free(nick);
}


void logout(gboolean reinit_events, gboolean gtk_alive)
{
if (sock>0)
	{
  ourstatus=S_OFFLINE;
  ourstatus_num=0;
  if (gtk_alive==TRUE) gtk_option_menu_set_history(GTK_OPTION_MENU(statusmenu),ourstatus_num);
  send_status(ourstatus, ourtext->str, ourpri, ouricon);
  send_data("</jabber>");
  close(sock);
  gdk_input_remove(tag);
  sock=-1;
  events_free();
  if (reinit_events==TRUE) events_init();
  if (gtk_alive==TRUE)
		{
		gtk_widget_set_sensitive(gtk_item_factory_get_widget(item_factory, "<main>/User/Login"),TRUE);
 		gtk_widget_set_sensitive(gtk_item_factory_get_widget(item_factory, "<main>/User/Logout"),FALSE);
		}
  printf("\nClosing connection... Ok");
  fflush(stdout);
  }
else
 {
 events_free();
 if (reinit_events==TRUE) events_init();
 }
}
