#include <gtk/gtk.h>
#include <stdio.h>
#include <time.h>
#include <glib.h>

#ifdef GNOME
  #include <gnome.h>
#endif

#include "common.h"
#include "main.h"
#include "miscwin.h"
#include "gui.h"
#include "menu.h"
#include "xpm.h"
#include "events.h"
#include "mpair.h"
#include "ejpair.h"
#include "readwin.h"
#include "sendwin.h"

GtkWidget*
create_win ()
{
  GtkWidget *vbox1;
  GtkWidget *hbox1;
  GtkWidget *optionmenu1_menu;
  GtkWidget *but;
  GtkWidget *glade_menuitem;
  GtkWidget *handlebox;

  GdkPixmap *iconpixmap;
  GdkBitmap *iconmask;

#ifndef GNOME
  win = gtk_window_new (GTK_WINDOW_TOPLEVEL);
#else
  win = gnome_app_new ("cabbar", "cabbar");
#endif
  gtk_window_set_title (GTK_WINDOW (win), "cabbar");
  gtk_window_set_policy (GTK_WINDOW (win), TRUE, TRUE, FALSE);
  gtk_widget_realize (win);
  gtk_widget_set_usize(win, 160, 280);
  gtk_signal_connect (GTK_OBJECT(win),"delete_event",GTK_SIGNAL_FUNC(delete_event),NULL);
  gtk_signal_connect (GTK_OBJECT(win),"destroy",GTK_SIGNAL_FUNC(destroy),NULL);

  iconpixmap = gdk_pixmap_create_from_xpm_d(win->window, &iconmask,&win->style->bg[GTK_STATE_NORMAL],logo2_xpm);
  gdk_window_set_icon(win->window,NULL,iconpixmap,iconmask);

  gdk_window_set_icon_name(win->window,"cabbar");


  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox1);

#ifndef GNOME
  gtk_container_add (GTK_CONTAINER (win), vbox1);
#else
  gnome_app_set_contents (GNOME_APP(win), vbox1);
#endif

  handlebox = gtk_handle_box_new();
  gtk_widget_show(handlebox);
  gtk_box_pack_start(GTK_BOX(vbox1), handlebox, FALSE, FALSE, 0);
  gtk_container_add (GTK_CONTAINER(handlebox), create_menu(win));

  notebook = gtk_notebook_new ();
  gtk_notebook_set_show_border (GTK_NOTEBOOK (notebook), FALSE);
  gtk_notebook_set_scrollable (GTK_NOTEBOOK (notebook), TRUE);
  gtk_notebook_set_tab_hborder (GTK_NOTEBOOK (notebook), 1);
  gtk_notebook_set_tab_vborder (GTK_NOTEBOOK (notebook), 1);
  gtk_notebook_popup_enable (GTK_NOTEBOOK (notebook));
  gtk_box_pack_start (GTK_BOX (vbox1), notebook, TRUE, TRUE, 0);
  gtk_widget_show (notebook);
  gtk_signal_connect(GTK_OBJECT(notebook),"switch_page",GTK_SIGNAL_FUNC(switch_page),NULL);

  hbox1 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox1);
  gtk_box_pack_start (GTK_BOX (vbox1), hbox1, FALSE, TRUE, 0);

  statusmenu = gtk_option_menu_new ();
  gtk_widget_show (statusmenu);
  gtk_box_pack_start (GTK_BOX (hbox1), statusmenu, TRUE, TRUE, 0);
  gtk_widget_set_usize (statusmenu, -1, 25);
  optionmenu1_menu = gtk_menu_new ();

  glade_menuitem=gtk_menu_item_new();
  gtk_container_add(GTK_CONTAINER(glade_menuitem),create_status_content(online_xpm,"Online"));
  gtk_widget_show (glade_menuitem);
  gtk_signal_connect_object(GTK_OBJECT(glade_menuitem),"activate",GTK_SIGNAL_FUNC(Status_Change),(gpointer)0);
  gtk_menu_append (GTK_MENU (optionmenu1_menu), glade_menuitem);

  glade_menuitem=gtk_menu_item_new();
  gtk_container_add(GTK_CONTAINER(glade_menuitem),create_status_content(offline_xpm,"Offline"));
  gtk_widget_show (glade_menuitem);
  gtk_signal_connect_object(GTK_OBJECT(glade_menuitem),"activate",GTK_SIGNAL_FUNC(Status_Change),(gpointer)1);
  gtk_menu_append (GTK_MENU (optionmenu1_menu), glade_menuitem);


  gtk_option_menu_set_menu (GTK_OPTION_MENU (statusmenu), optionmenu1_menu);

  but = gtk_button_new_with_label ("...");
  gtk_widget_show (but);
  gtk_box_pack_start (GTK_BOX (hbox1), but, FALSE, FALSE, 0);
  gtk_widget_set_usize (but, 20, 20);
  gtk_signal_connect(GTK_OBJECT(but),"clicked",GTK_SIGNAL_FUNC(create_extstatuswin),NULL);

  create_popup_menu();

  return win;
}


void create_rostwin ()
{
  GtkWidget *vbox1;
  GtkWidget *table1;
  GtkWidget *l;

  rostwin.win = gtk_window_new (GTK_WINDOW_DIALOG);
/*  gtk_container_border_width (GTK_CONTAINER (rostwin.win), 4); */
  gtk_window_set_title (GTK_WINDOW (rostwin.win), "Roster functions");
  gtk_window_set_policy (GTK_WINDOW (rostwin.win), TRUE, TRUE, FALSE);

  gtk_signal_connect(GTK_OBJECT(rostwin.win),"delete_event",GTK_SIGNAL_FUNC(delete_event),NULL);
  gtk_signal_connect(GTK_OBJECT(rostwin.win),"destroy",GTK_SIGNAL_FUNC(destroy),NULL);

  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox1);
  gtk_container_add (GTK_CONTAINER (rostwin.win), vbox1);

  table1 = gtk_table_new (2, 2, FALSE);
  gtk_widget_show (table1);
  gtk_box_pack_start (GTK_BOX (vbox1), table1, FALSE, TRUE, 0);

  rostwin.optmenu = gtk_option_menu_new ();
  gtk_widget_show (rostwin.optmenu);
  gtk_table_attach (GTK_TABLE (table1), rostwin.optmenu, 1, 2, 0, 1,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, (GtkAttachOptions) GTK_FILL, 0, 0);
  gtk_widget_set_usize (rostwin.optmenu, -1, 25);

  rostwin.optmenu_menu = gtk_menu_new ();

  rostwin.optmenu_item[0] = gtk_menu_item_new_with_label ("Add User");
  gtk_widget_show (rostwin.optmenu_item[0]);
  gtk_menu_append (GTK_MENU (rostwin.optmenu_menu), rostwin.optmenu_item[0]);

  rostwin.optmenu_item[1] = gtk_menu_item_new_with_label ("Remove User");
  gtk_widget_show (rostwin.optmenu_item[1]);
  gtk_menu_append (GTK_MENU (rostwin.optmenu_menu), rostwin.optmenu_item[1]);

  rostwin.optmenu_item[2] = gtk_menu_item_new_with_label ("Edit User");
  gtk_widget_show (rostwin.optmenu_item[2]);
  gtk_menu_append (GTK_MENU (rostwin.optmenu_menu), rostwin.optmenu_item[2]);

  rostwin.optmenu_item[3] = gtk_menu_item_new_with_label ("Add Group");
  gtk_widget_show (rostwin.optmenu_item[3]);
  gtk_menu_append (GTK_MENU (rostwin.optmenu_menu), rostwin.optmenu_item[3]);

  rostwin.optmenu_item[4] = gtk_menu_item_new_with_label ("Remove Group");
  gtk_widget_show (rostwin.optmenu_item[4]);
  gtk_menu_append (GTK_MENU (rostwin.optmenu_menu), rostwin.optmenu_item[4]);

  rostwin.optmenu_item[5] = gtk_menu_item_new_with_label ("Edit Group");
  gtk_widget_show (rostwin.optmenu_item[5]);
  gtk_menu_append (GTK_MENU (rostwin.optmenu_menu), rostwin.optmenu_item[5]);

  gtk_signal_connect_object(GTK_OBJECT(rostwin.optmenu_item[0]),"activate",GTK_SIGNAL_FUNC(rostwin_FuncChange),(gpointer)0);
  gtk_signal_connect_object(GTK_OBJECT(rostwin.optmenu_item[1]),"activate",GTK_SIGNAL_FUNC(rostwin_FuncChange),(gpointer)1);
  gtk_signal_connect_object(GTK_OBJECT(rostwin.optmenu_item[2]),"activate",GTK_SIGNAL_FUNC(rostwin_FuncChange),(gpointer)2);
  gtk_signal_connect_object(GTK_OBJECT(rostwin.optmenu_item[3]),"activate",GTK_SIGNAL_FUNC(rostwin_FuncChange),(gpointer)3);
  gtk_signal_connect_object(GTK_OBJECT(rostwin.optmenu_item[4]),"activate",GTK_SIGNAL_FUNC(rostwin_FuncChange),(gpointer)4);
  gtk_signal_connect_object(GTK_OBJECT(rostwin.optmenu_item[5]),"activate",GTK_SIGNAL_FUNC(rostwin_FuncChange),(gpointer)5);

  gtk_option_menu_set_menu (GTK_OPTION_MENU(rostwin.optmenu), rostwin.optmenu_menu);


  rostwin.entry = gtk_entry_new();
  gtk_widget_show (rostwin.entry);
  gtk_table_attach (GTK_TABLE (table1), rostwin.entry, 1, 2, 1, 2,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, (GtkAttachOptions) GTK_FILL, 0, 0);
  gtk_widget_set_usize (rostwin.entry, -1, 22);

  l = gtk_label_new ("Action: ");
  gtk_widget_show (l);
  gtk_table_attach (GTK_TABLE (table1), l, 0, 1, 0, 1,
                    (GtkAttachOptions) GTK_FILL, (GtkAttachOptions) GTK_FILL, 3, 0);
  gtk_label_set_justify (GTK_LABEL (l), GTK_JUSTIFY_RIGHT);
  gtk_misc_set_alignment (GTK_MISC (l), 1, 0.5);

  l = gtk_label_new ("Name of User/Group:");
  gtk_widget_show (l);
  gtk_table_attach (GTK_TABLE (table1), l, 0, 1, 1, 2,
                    (GtkAttachOptions) GTK_FILL, (GtkAttachOptions) GTK_FILL, 3, 0);
  gtk_label_set_justify (GTK_LABEL (l), GTK_JUSTIFY_RIGHT);
  gtk_misc_set_alignment (GTK_MISC (l), 1, 0.5);

{
  GtkWidget *hbox;

  hbox = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox);

  rostwin.cancel = gtk_button_new_with_label ("Cancel");
  gtk_widget_show (rostwin.cancel);
  gtk_box_pack_end (GTK_BOX (hbox), rostwin.cancel, FALSE, TRUE, 0);
  gtk_widget_set_usize (rostwin.cancel, 60, 25);

  rostwin.ok = gtk_button_new_with_label ("Ok");
  gtk_widget_show (rostwin.ok);
  gtk_box_pack_end (GTK_BOX (hbox), rostwin.ok, FALSE, TRUE, 0);
  gtk_widget_set_usize (rostwin.ok, 60, 25);

  gtk_box_pack_start(GTK_BOX (vbox1), hbox, FALSE, TRUE, 0);
}

  gtk_signal_connect_object(GTK_OBJECT(rostwin.cancel),"clicked",GTK_SIGNAL_FUNC(gtk_widget_destroy),(gpointer)rostwin.win);
  gtk_signal_connect(GTK_OBJECT(rostwin.ok),"clicked",GTK_SIGNAL_FUNC(rostwin_ok),NULL);

  gtk_widget_set_sensitive(rostwin.optmenu,FALSE);
  gtk_widget_show(rostwin.win);

  gtk_window_set_modal(GTK_WINDOW(rostwin.win),TRUE);
}

void create_statuswin()
{
  GtkWidget *vbox1;

  statuswin.win = gtk_window_new (GTK_WINDOW_DIALOG);
  gtk_container_border_width (GTK_CONTAINER (statuswin.win), 3);
  gtk_window_set_title (GTK_WINDOW (statuswin.win), "Status Change");
  gtk_window_position (GTK_WINDOW (statuswin.win), GTK_WIN_POS_CENTER);
  gtk_window_set_policy (GTK_WINDOW (statuswin.win), TRUE, FALSE, FALSE);
  gtk_signal_connect (GTK_OBJECT(statuswin.win),"delete_event",GTK_SIGNAL_FUNC(delete_event),NULL);
  gtk_signal_connect (GTK_OBJECT(statuswin.win),"destroy",GTK_SIGNAL_FUNC(statuswin_Destroy),NULL);

  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox1);
  gtk_container_add (GTK_CONTAINER (statuswin.win), vbox1);

  statuswin.label = gtk_label_new ("Tell us more about your status:");
  gtk_widget_show (statuswin.label);
  gtk_box_pack_start (GTK_BOX (vbox1), statuswin.label, FALSE, TRUE, 0);
  gtk_label_set_justify (GTK_LABEL (statuswin.label), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment (GTK_MISC (statuswin.label), 0, 0.5);

  statuswin.entry = gtk_entry_new ();
  gtk_widget_show (statuswin.entry);
  gtk_box_pack_start (GTK_BOX (vbox1), statuswin.entry, FALSE, TRUE, 0);
  gtk_widget_set_usize (statuswin.entry, 230, -1);
  gtk_entry_set_text (GTK_ENTRY (statuswin.entry), "This is my status");


{
  GtkWidget *hbox;

  hbox = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox);

  statuswin.button_cancel = gtk_button_new_with_label ("Cancel");
  gtk_widget_show (statuswin.button_cancel);
  gtk_box_pack_end (GTK_BOX (hbox), statuswin.button_cancel, FALSE, TRUE, 0);
  gtk_widget_set_usize (statuswin.button_cancel, 60, 25);

  statuswin.button_ok = gtk_button_new_with_label ("Ok");
  gtk_widget_show (statuswin.button_ok);
  gtk_box_pack_end (GTK_BOX (hbox), statuswin.button_ok, FALSE, TRUE, 0);
  gtk_widget_set_usize (statuswin.button_ok, 60, 25);

  gtk_box_pack_start(GTK_BOX (vbox1), hbox, FALSE, TRUE, 0);
}

  gtk_signal_connect_object(GTK_OBJECT(statuswin.button_cancel),"clicked",GTK_SIGNAL_FUNC(gtk_widget_destroy),(gpointer)statuswin.win);
  gtk_signal_connect_object(GTK_OBJECT(statuswin.button_ok),"clicked",GTK_SIGNAL_FUNC(statuswin_Ok),NULL);

  gtk_widget_show(statuswin.win);
  gtk_window_set_modal(GTK_WINDOW(statuswin.win),TRUE);
}


void create_extstatuswin()
{
  GtkWidget *vbox1;
  GtkWidget *label;
  GtkWidget *table1;
  GtkWidget *option_status_menu;
  GtkWidget *glade_menuitem;
  GtkWidget *option_icon_menu;
  GtkObject *spin_pri_adj;

  extstatuswin.win = gtk_window_new (GTK_WINDOW_DIALOG);
  gtk_container_border_width (GTK_CONTAINER (extstatuswin.win), 3);
  gtk_window_set_title (GTK_WINDOW (extstatuswin.win), "Extended Status Change");
  gtk_window_position (GTK_WINDOW (extstatuswin.win), GTK_WIN_POS_CENTER);
  gtk_window_set_policy (GTK_WINDOW (extstatuswin.win), TRUE, TRUE, TRUE);
  gtk_signal_connect (GTK_OBJECT(extstatuswin.win),"delete_event",GTK_SIGNAL_FUNC(delete_event),NULL);

  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox1);
  gtk_container_add (GTK_CONTAINER (extstatuswin.win), vbox1);

  table1 = gtk_table_new (4, 2, FALSE);
  gtk_widget_show (table1);
  gtk_box_pack_start (GTK_BOX (vbox1), table1, TRUE, TRUE, 0);

  label = gtk_label_new ("Status:");
  gtk_widget_show (label);
  gtk_table_attach (GTK_TABLE (table1), label, 0, 1, 0, 1,
                    (GtkAttachOptions) GTK_FILL, (GtkAttachOptions) GTK_FILL, 0, 0);
  gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_RIGHT);
  gtk_misc_set_alignment (GTK_MISC (label), 1, 0.5);
  gtk_misc_set_padding (GTK_MISC (label), 3, 0);

  extstatuswin.option_status = gtk_option_menu_new ();
  gtk_widget_show (extstatuswin.option_status);
  gtk_table_attach (GTK_TABLE (table1), extstatuswin.option_status, 1, 2, 0, 1,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, (GtkAttachOptions) GTK_FILL, 0, 0);
  gtk_widget_set_usize (extstatuswin.option_status, -1, 25);
  option_status_menu = gtk_menu_new ();

  glade_menuitem=gtk_menu_item_new();
  gtk_container_add(GTK_CONTAINER(glade_menuitem),create_status_content(online_xpm,"Online"));
  gtk_widget_show (glade_menuitem);
  gtk_signal_connect_object(GTK_OBJECT(glade_menuitem),"activate",GTK_SIGNAL_FUNC(ExtStatus_Change),(gpointer)0);
  gtk_menu_append (GTK_MENU(option_status_menu), glade_menuitem);

  glade_menuitem=gtk_menu_item_new();
  gtk_container_add(GTK_CONTAINER(glade_menuitem),create_status_content(offline_xpm,"Offline"));
  gtk_widget_show (glade_menuitem);
  gtk_signal_connect_object(GTK_OBJECT(glade_menuitem),"activate",GTK_SIGNAL_FUNC(ExtStatus_Change),(gpointer)1);
  gtk_menu_append (GTK_MENU(option_status_menu), glade_menuitem);

  gtk_option_menu_set_menu (GTK_OPTION_MENU (extstatuswin.option_status), option_status_menu);

  extstatuswin.ent_desc = gtk_entry_new ();
  gtk_entry_set_text(GTK_ENTRY(extstatuswin.ent_desc),ourtext->str);
  gtk_widget_show (extstatuswin.ent_desc);
  gtk_table_attach (GTK_TABLE (table1), extstatuswin.ent_desc, 1, 2, 3, 4,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, (GtkAttachOptions) GTK_FILL, 0, 0);

  extstatuswin.option_icon = gtk_option_menu_new ();
  gtk_widget_show (extstatuswin.option_icon);
  gtk_table_attach (GTK_TABLE (table1), extstatuswin.option_icon, 1, 2, 1, 2,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, (GtkAttachOptions) GTK_FILL, 0, 0);
  gtk_widget_set_usize (extstatuswin.option_icon, -1, 25);
  option_icon_menu = gtk_menu_new ();

  glade_menuitem=gtk_menu_item_new();
  gtk_container_add(GTK_CONTAINER(glade_menuitem),create_status_content(online_xpm,"Normal"));
  gtk_widget_show (glade_menuitem);
  gtk_signal_connect_object(GTK_OBJECT(glade_menuitem),"activate",GTK_SIGNAL_FUNC(ExtStatus_ChangeIcon),(gpointer)0);
  gtk_menu_append (GTK_MENU(option_icon_menu), glade_menuitem);

  glade_menuitem=gtk_menu_item_new();
  gtk_container_add(GTK_CONTAINER(glade_menuitem),create_status_content(away_xpm,"Away"));
  gtk_widget_show (glade_menuitem);
  gtk_signal_connect_object(GTK_OBJECT(glade_menuitem),"activate",GTK_SIGNAL_FUNC(ExtStatus_ChangeIcon),(gpointer)1);
  gtk_menu_append (GTK_MENU(option_icon_menu), glade_menuitem);

  gtk_option_menu_set_menu (GTK_OPTION_MENU (extstatuswin.option_icon), option_icon_menu);

  label = gtk_label_new ("Icon:");
  gtk_widget_show (label);
  gtk_table_attach (GTK_TABLE (table1), label, 0, 1, 1, 2,
                    (GtkAttachOptions) GTK_FILL, (GtkAttachOptions) GTK_FILL, 0, 0);
  gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_RIGHT);
  gtk_misc_set_alignment (GTK_MISC (label), 1, 0.5);
  gtk_misc_set_padding (GTK_MISC (label), 3, 0);

  label = gtk_label_new ("Priority:");
  gtk_widget_show (label);
  gtk_table_attach (GTK_TABLE (table1), label, 0, 1, 2, 3,
                    (GtkAttachOptions) GTK_FILL, (GtkAttachOptions) GTK_FILL, 0, 0);
  gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_RIGHT);
  gtk_misc_set_alignment (GTK_MISC (label), 1, 0.5);
  gtk_misc_set_padding (GTK_MISC (label), 3, 0);

  label = gtk_label_new ("Description:");
  gtk_widget_show (label);
  gtk_table_attach (GTK_TABLE (table1), label, 0, 1, 3, 4,
                    (GtkAttachOptions) GTK_FILL, (GtkAttachOptions) GTK_FILL, 0, 0);
  gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_RIGHT);
  gtk_misc_set_alignment (GTK_MISC (label), 1, 0.5);
  gtk_misc_set_padding (GTK_MISC (label), 3, 0);

  spin_pri_adj = gtk_adjustment_new (1, -255, 255, 1, 10, 10);
  extstatuswin.spin_pri = gtk_spin_button_new (GTK_ADJUSTMENT (spin_pri_adj), 1, 0);
  gtk_widget_show (extstatuswin.spin_pri);
  gtk_table_attach (GTK_TABLE (table1), extstatuswin.spin_pri, 1, 2, 2, 3,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, (GtkAttachOptions) GTK_FILL, 0, 0);
  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (extstatuswin.spin_pri), TRUE);

{
  GtkWidget *hbox;

  hbox = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox);

  extstatuswin.button_cancel = gtk_button_new_with_label ("Cancel");
  gtk_widget_show (extstatuswin.button_cancel);
  gtk_box_pack_end (GTK_BOX (hbox), extstatuswin.button_cancel, FALSE, TRUE, 0);
  gtk_widget_set_usize (extstatuswin.button_cancel, 60, 25);

  extstatuswin.button_ok = gtk_button_new_with_label ("Ok");
  gtk_widget_show (extstatuswin.button_ok);
  gtk_box_pack_end (GTK_BOX (hbox), extstatuswin.button_ok, FALSE, TRUE, 0);
  gtk_widget_set_usize (extstatuswin.button_ok, 60, 25);

  gtk_box_pack_start(GTK_BOX (vbox1), hbox, FALSE, TRUE, 0);
}

  gtk_signal_connect_object(GTK_OBJECT(extstatuswin.button_cancel),"clicked",GTK_SIGNAL_FUNC(gtk_widget_destroy),(gpointer)extstatuswin.win);
  gtk_signal_connect_object(GTK_OBJECT(extstatuswin.button_ok),"clicked",GTK_SIGNAL_FUNC(extstatuswin_Ok),NULL);

  gtk_option_menu_set_history(GTK_OPTION_MENU(extstatuswin.option_status),ourstatus_num);
  extstatuswin.status=ourstatus_num;

  gtk_option_menu_set_history(GTK_OPTION_MENU(extstatuswin.option_icon),ouricon_num);
  extstatuswin.icon=ouricon_num;

  gtk_spin_button_set_value(GTK_SPIN_BUTTON(extstatuswin.spin_pri),ourpri);

  gtk_widget_show(extstatuswin.win);
  gtk_window_set_modal(GTK_WINDOW(extstatuswin.win),TRUE);
}



void create_debugwin()
{
  GtkWidget *vbox1;
  GtkWidget *hbox1;

  
  debug.win = gtk_window_new (GTK_WINDOW_DIALOG);
  gtk_container_border_width (GTK_CONTAINER (debug.win), 2);
  gtk_window_set_title (GTK_WINDOW (debug.win), "Debug Window");
  gtk_window_set_policy (GTK_WINDOW (debug.win), TRUE, TRUE, FALSE);
  gtk_signal_connect (GTK_OBJECT(debug.win),"delete_event",GTK_SIGNAL_FUNC(hide_event),NULL);
  gtk_signal_connect_object(GTK_OBJECT(debug.win),"destroy",GTK_SIGNAL_FUNC(gtk_widget_hide),(gpointer)debug.win);

  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox1);
  gtk_container_add (GTK_CONTAINER (debug.win), vbox1);

  hbox1 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox1);
  gtk_box_pack_start (GTK_BOX (vbox1), hbox1, TRUE, TRUE, 0);

  debug.text = gtk_text_new (NULL, NULL);
  gtk_widget_show (debug.text);
  gtk_box_pack_start (GTK_BOX (hbox1), debug.text, TRUE, TRUE, 0);
  gtk_text_set_editable(GTK_TEXT(debug.text),TRUE);

  debug.scroll = gtk_vscrollbar_new(GTK_TEXT(debug.text)->vadj);
  gtk_widget_show (debug.scroll);
  gtk_box_pack_start (GTK_BOX (hbox1), debug.scroll, FALSE, TRUE, 0);

  debug.button = gtk_button_new_with_label ("Receive");
  gtk_widget_show (debug.button);
  gtk_box_pack_start (GTK_BOX (vbox1), debug.button, FALSE, TRUE, 0);
  gtk_signal_connect_object(GTK_OBJECT(debug.button),"clicked",GTK_SIGNAL_FUNC(add_debug),NULL);
}

void create_loginwin()
{
  GtkWidget *vbox1;
  GtkWidget *table1;
  GtkWidget *label;

  loginwin.win = gtk_window_new (GTK_WINDOW_DIALOG);
  gtk_container_border_width (GTK_CONTAINER (loginwin.win), 3);
  gtk_window_set_title (GTK_WINDOW (loginwin.win), "Login to Server");
  gtk_window_set_policy (GTK_WINDOW (loginwin.win), TRUE, TRUE, TRUE);

  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox1);
  gtk_container_add (GTK_CONTAINER (loginwin.win), vbox1);

  table1 = gtk_table_new (3, 2, FALSE);
  gtk_widget_show (table1);
  gtk_box_pack_start (GTK_BOX (vbox1), table1, TRUE, TRUE, 0);

  label = gtk_label_new ("UserID:");
  gtk_widget_show (label);
  gtk_table_attach (GTK_TABLE (table1), label, 0, 1, 0, 1,
                    (GtkAttachOptions) GTK_FILL, (GtkAttachOptions) GTK_FILL, 0, 0);
  gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_RIGHT);
  gtk_misc_set_alignment (GTK_MISC (label), 1, 0.5);

  label = gtk_label_new ("Password:");
  gtk_widget_show (label);
  gtk_table_attach (GTK_TABLE (table1), label, 0, 1, 1, 2,
                    (GtkAttachOptions) GTK_FILL, (GtkAttachOptions) GTK_FILL, 0, 0);
  gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_RIGHT);
  gtk_misc_set_alignment (GTK_MISC (label), 1, 0.5);

  label = gtk_label_new ("Nickname:");
  gtk_widget_show (label);
  gtk_table_attach (GTK_TABLE (table1), label, 0, 1, 2, 3,
                    (GtkAttachOptions) GTK_FILL, (GtkAttachOptions) GTK_FILL, 0, 0);
  gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_RIGHT);
  gtk_misc_set_alignment (GTK_MISC (label), 1, 0.5);

  loginwin.ent_uid = gtk_entry_new ();
  gtk_widget_show (loginwin.ent_uid);
  gtk_table_attach (GTK_TABLE (table1), loginwin.ent_uid, 1, 2, 0, 1,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, (GtkAttachOptions) GTK_FILL, 0, 0);

  loginwin.ent_pass = gtk_entry_new ();
  gtk_widget_show (loginwin.ent_pass);
  gtk_table_attach (GTK_TABLE (table1), loginwin.ent_pass, 1, 2, 1, 2,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, (GtkAttachOptions) GTK_FILL, 0, 0);
  gtk_entry_set_visibility (GTK_ENTRY (loginwin.ent_pass), FALSE);

  loginwin.ent_nick = gtk_entry_new ();
  gtk_widget_show (loginwin.ent_nick);
  gtk_table_attach (GTK_TABLE (table1), loginwin.ent_nick, 1, 2, 2, 3,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, (GtkAttachOptions) GTK_FILL, 0, 0);

{
  GtkWidget *hbox;

  hbox = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox);

  loginwin.cancel = gtk_button_new_with_label ("Cancel");
  gtk_widget_show (loginwin.cancel);
  gtk_box_pack_end (GTK_BOX (hbox), loginwin.cancel, FALSE, TRUE, 0);
  gtk_widget_set_usize (loginwin.cancel, 60, 25);

  loginwin.ok = gtk_button_new_with_label ("Ok");
  gtk_widget_show (loginwin.ok);
  gtk_box_pack_end (GTK_BOX (hbox), loginwin.ok, FALSE, TRUE, 0);
  gtk_widget_set_usize (loginwin.ok, 60, 25);

  gtk_box_pack_start(GTK_BOX (vbox1), hbox, FALSE, TRUE, 0);
}

  gtk_signal_connect_object(GTK_OBJECT(loginwin.cancel),"clicked",GTK_SIGNAL_FUNC(gtk_widget_destroy),(gpointer)loginwin.win);
  gtk_signal_connect(GTK_OBJECT(loginwin.ok),"clicked",GTK_SIGNAL_FUNC(loginwin_Ok),NULL);

{
char *s;

	s=ejpair_getval(cfg,"user");
  if (s!=NULL) gtk_entry_set_text(GTK_ENTRY(loginwin.ent_uid), s);
	s=ejpair_getval(cfg,"pass");
 	if (s!=NULL) gtk_entry_set_text(GTK_ENTRY(loginwin.ent_pass), s);
	s=ejpair_getval(cfg,"nick");
  if (s!=NULL) gtk_entry_set_text(GTK_ENTRY(loginwin.ent_nick), s);
}

  gtk_widget_show(loginwin.win);
  gtk_window_set_modal(GTK_WINDOW(loginwin.win),TRUE);
}
