
/*
stolen from jpair.c in the Jabber project (www.jabber.org)
the difference is, value is void *, instead of char *
also, mpair.c implements mpair_remove, and ->prev

uses strdup to duplicate void pointers, so be careful ;) (marked with FIXME)

disq <disq@iname.com>
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "mpair.h"

mpair *mpair_append(mpair *j1, mpair *j2)
{
	mpair *j_count = j1;

	if(j1 == NULL)
		return j2;
	if(j2 == NULL)
		return j1;
	while(j_count->next != NULL)
		j_count = j_count->next;
	j_count->next = j2;
	j2->prev = j_count;
	return j1;
}

mpair *mpair_add(mpair *p, char *key, void *val)
{
	mpair *p_new;

	p_new = malloc(sizeof(mpair));
	p_new->val = val;
	p_new->key = key;
	p_new->prev = NULL;
	p_new->next = NULL;

	if(p == NULL)
		return p_new;
	p_new->next = p;
	p->prev = p_new;
	return p_new;
}

mpair *mpair_new(mpair *p, char *key, void *val, int flag)
{
	mpair *p_new;

	p_new = malloc(sizeof(mpair));
	if(key == NULL)
		p_new->key = NULL;
	else
		p_new->key = strdup(key);

	if(val == NULL)
		p_new->val = NULL;
	else
		p_new->val = val;  /*strdup(val); FIXME */

	p_new->val = val;
	p_new->flag = flag;
	p_new->prev = NULL;
	p_new->next = NULL;

	if(p == NULL)
		return p_new;
	p_new->next = p;
	p->prev = p_new;

	return p_new;
}

void free_mpair(mpair *p)
{
	mpair *next;

	next = p;

	while(next != NULL)
	{
		p = next;
		next = p->next;
		free(p->key);
/*		free(p->val); FIXME*/
		free(p);
	}
}

mpair *mpair_dup(mpair *p)
{
	mpair *new = NULL;

	while(p != NULL)
	{
		new = mpair_add(new,NULL,NULL);
		if(p->key == NULL)
			new->key = p->key;
		else
			new->key = strdup(p->key);

		if(p->val == NULL)
			new->val = p->val;
		else
			new->val = p->val; /*strdup(p->val); FIXME*/

		new->flag = p->flag;
		p = p->next;
	}
	return new;
}

mpair *mpair_get(mpair *p, char *key)
{
	while(p != NULL)
	{
		if(p->key != NULL && key != NULL && strcasecmp(p->key,key) == 0)
			return p;
		p = p->next;
	}
	return NULL;
}

void *mpair_getval(mpair *p, char *key)
{
	while(p != NULL)
	{
		if(p->key != NULL && key != NULL && strcasecmp(p->key,key) == 0)
			return p->val;
		p = p->next;
	}
	return NULL;
}

void *mpair_getvalnew(mpair *p, char *key)
{
	char *val;

	val = mpair_getval(p, key);
	if(val != NULL)
		return val; /*strdup(val); FIXME*/
	return NULL; 
}

int mpair_getflag(mpair *p, char *key)
{
	while(p != NULL)
	{
		if(p->key != NULL && key != NULL && strcasecmp(p->key,key) == 0)
			return p->flag;
		p = p->next;
	}
	return 0;
}

void mpair_print(mpair *p)
{
	printf("===MPAIR===[");
	while(p != NULL)
	{
		printf("%s[%s] ",p->key, (char *)p->val);
		p = p->next;
	}
	printf("]===/MPAIR===\n");
}


mpair *mpair_remove(mpair *p, char *key, int dofree)
{
mpair *dup; /* p clone */
mpair *a, *b, *c; /* just a fix temp vars */

dup=mpair_dup(p);
if (dofree!=0) free_mpair(p);

a=mpair_get(dup,key);
if (a==NULL) return(dup);
else if (a->prev==NULL)
  {
  b=a->next;
  if (dofree!=0)
	{
	free(a->key);
/*	free(a->val); FIXME*/
	free(a);
	}
  return(b);
  }
else if (a->next==NULL)
  {
  b=a->prev;
  b->next=NULL;
  if (dofree!=0)
	{
	free(a->key);
/*	free(a->val); FIXME*/
	free(a);
	}
  return(dup);
  }
else
  {
  b=a->prev;
  c=a->next;
  if (dofree!=0)
	{
	free(a->key);
/*	free(a->val); FIXME*/
	free(a);
	}
  b->next=c;
  c->prev=b;
  return(dup);
  }
return(NULL); /* won't reach here, this line is just to suppress warning */
}
