#include <gtk/gtk.h>
#include <stdio.h>
#include <time.h>
#include <glib.h>

#ifdef GNOME
  #include <gnome.h>
#endif

#include "readwin.h"
#include "sendwin.h"
#include "gui.h"
#include "menu.h"
#include "events.h"
#include "mpair.h"
#include "xpm.h"
#include "main.h"
#include "prefswin.h"


readwin_struct *open_readwin(userstruct *u)
{
  GtkWidget *vbox1;
  GtkWidget *hbox1;
  GtkWidget *hbox2;
  GtkWidget *vscrollbar1;
  char s[4096];
  readwin_struct *rw;

  if (u->readwin!=NULL)
	{
	puts("one read window open already");
	return(NULL);
	}

  if (u->msgs==NULL)
	{
	printf("\nno messages from user in buffer (%d)\n",u->num_msgs);
	fflush(stdout);
	return(NULL);
	}

  rw=g_malloc(sizeof(readwin_struct));
  rw->max=u->num_msgs;
  rw->u=u;
  u->readwin=(gpointer)rw;

  rw->win = gtk_window_new (GTK_WINDOW_DIALOG);
  sprintf(s,"Read Message (%s)",u->uname->str);
  gtk_window_set_title (GTK_WINDOW (rw->win), s);
  gtk_window_position (GTK_WINDOW (rw->win), GTK_WIN_POS_CENTER);
  gtk_window_set_policy (GTK_WINDOW (rw->win), TRUE, TRUE, FALSE);
  gtk_widget_set_usize(rw->win, 300, 180);
  gtk_signal_connect_object(GTK_OBJECT(rw->win),"destroy",GTK_SIGNAL_FUNC(readwin_close),(gpointer)rw);
  gtk_signal_connect(GTK_OBJECT(rw->win),"delete_event",GTK_SIGNAL_FUNC(readwin_delete_event),NULL);

  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox1);
  gtk_container_add (GTK_CONTAINER (rw->win), vbox1);
  gtk_container_border_width (GTK_CONTAINER (vbox1), 3);

  hbox1 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox1);
  gtk_box_pack_start (GTK_BOX (vbox1), hbox1, TRUE, TRUE, 0);

  rw->text = gtk_text_new (NULL, NULL);
  gtk_widget_show (rw->text);
  gtk_box_pack_start (GTK_BOX (hbox1), rw->text, TRUE, TRUE, 0);
  gtk_text_set_editable (GTK_TEXT (rw->text), FALSE);

  vscrollbar1 = gtk_vscrollbar_new (GTK_TEXT(rw->text)->vadj);
  gtk_widget_show (vscrollbar1);
  gtk_box_pack_start (GTK_BOX (hbox1), vscrollbar1, FALSE, TRUE, 0);

  hbox2 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox2);
  gtk_box_pack_start(GTK_BOX (vbox1), hbox2, FALSE, TRUE, 0);

  rw->but_close = gtk_button_new_with_label ("Close");
  gtk_widget_show (rw->but_close);
  gtk_box_pack_end (GTK_BOX (hbox2), rw->but_close, FALSE, TRUE, 0);
  gtk_widget_set_usize (rw->but_close, 60, 25);
  gtk_signal_connect_object(GTK_OBJECT(rw->but_close),"clicked",GTK_SIGNAL_FUNC(gtk_widget_destroy),(gpointer)(rw->win));

  rw->but_next = gtk_button_new_with_label ("Next");
  gtk_widget_show (rw->but_next);
  gtk_box_pack_end (GTK_BOX (hbox2), rw->but_next, FALSE, TRUE, 0);
  gtk_widget_set_usize (rw->but_next, 60, 25);
  gtk_signal_connect_object(GTK_OBJECT(rw->but_next),"clicked",GTK_SIGNAL_FUNC(readwin_next),(gpointer)rw);

  rw->but_reply = gtk_button_new_with_label ("Reply");
  gtk_widget_show (rw->but_reply);
  gtk_box_pack_start (GTK_BOX (hbox2), rw->but_reply, FALSE, TRUE, 0);
  gtk_widget_set_usize (rw->but_reply, 60, 25);
  gtk_signal_connect_object(GTK_OBJECT(rw->but_reply),"clicked",GTK_SIGNAL_FUNC(readwin_reply),(gpointer)rw);

  rw->but_forward = gtk_button_new_with_label ("Forward");
  gtk_widget_show (rw->but_forward);
  gtk_box_pack_start (GTK_BOX (hbox2), rw->but_forward, FALSE, TRUE, 0);
  gtk_widget_set_usize (rw->but_reply, 60, 25);
  gtk_signal_connect_object(GTK_OBJECT(rw->but_forward),"clicked",GTK_SIGNAL_FUNC(readwin_forward),(gpointer)rw);

  readwin_update(rw);
  gtk_widget_show(rw->win);
  return(rw);
}


void readwin_update(readwin_struct *rw)
{
GdkFont *normfont;
GdkFont *boldfont;
msg_struct *m;

  if (rw->u->msgs==NULL)
	{
	puts("rw->u->msgs is NULL! (1)");
	return;
	}
  m=(msg_struct *)(rw->u->msgs->data);

  gtk_text_freeze (GTK_TEXT(rw->text));

  gtk_text_set_point(GTK_TEXT(rw->text), 0);
  gtk_text_forward_delete(GTK_TEXT(rw->text), gtk_text_get_length(GTK_TEXT(rw->text)));

  normfont = gdk_font_load ("-adobe-helvetica-medium-r-normal--*-120-*-*-*-*-*-*");
  boldfont = gdk_font_load ("-adobe-courier-bold-r-normal--*-120-*-*-*-*-*-*");

  if ((m->fromid!=NULL) || (m->fromnick!=NULL))
	{
	gtk_text_insert(GTK_TEXT (rw->text), boldfont, NULL, NULL, 
			   "\n    From: ", -1);
	if (m->fromid!=NULL)
	  {
	  gtk_text_insert(GTK_TEXT (rw->text), normfont, NULL, NULL, 
			   m->fromid, -1);
	  }
	if (m->fromnick!=NULL)
	  {
	  gtk_text_insert(GTK_TEXT (rw->text), normfont, NULL, NULL, 
			   " (", -1);
	  gtk_text_insert(GTK_TEXT (rw->text), normfont, NULL, NULL, 
			   m->fromnick, -1);
	  gtk_text_insert(GTK_TEXT (rw->text), normfont, NULL, NULL, 
			   ")", -1);
	  }
	}
	
  if (m->thread!=NULL)
	{
	gtk_text_insert(GTK_TEXT (rw->text), boldfont, NULL, NULL, 
			   "\n  Thread: ", -1);
	gtk_text_insert(GTK_TEXT (rw->text), normfont, NULL, NULL, 
			   m->thread, -1);
	}

  if (m->subj!=NULL)
	{
	gtk_text_insert(GTK_TEXT (rw->text), boldfont, NULL, NULL, 
			   "\n Subject: ", -1);
	gtk_text_insert(GTK_TEXT (rw->text), normfont, NULL, NULL, 
			   m->subj, -1);
	}

  if (m->pri!=NULL)
	{
	gtk_text_insert(GTK_TEXT (rw->text), boldfont, NULL, NULL, 
			   "\nPriority: ", -1);
	gtk_text_insert(GTK_TEXT (rw->text), normfont, NULL, NULL, 
			   m->pri, -1);
	}

  if (m->date!=NULL)
	{
	gtk_text_insert(GTK_TEXT (rw->text), boldfont, NULL, NULL, 
			   "\n    Date: ", -1);
	gtk_text_insert(GTK_TEXT (rw->text), normfont, NULL, NULL, 
			   m->date, -1);
	}

  if (m->msg!=NULL)
	{
	gtk_text_insert(GTK_TEXT (rw->text), normfont, NULL, NULL, 
			   "\n\n", -1);
	gtk_text_insert(GTK_TEXT (rw->text), normfont, NULL, NULL, 
			   m->msg, -1);
	}
	
  gdk_font_unref(boldfont);
  gdk_font_unref(normfont);

  gtk_text_thaw (GTK_TEXT(rw->text));

if (rw->max==0)
  {
  fprintf(stderr,"\nrw->max is 0! (1)\n");
  fflush(stderr);
  }
if (rw->max<2) gtk_widget_set_sensitive(rw->but_next,FALSE);
		  else gtk_widget_set_sensitive(rw->but_next,TRUE);
}


void readwin_next(readwin_struct *rw)
{
userstruct *u;
msg_struct *m=NULL;

u=rw->u;

if (rw->max==0)
  {
  fprintf(stderr,"\nrw->max is 0! (2)\n");
  fflush(stderr);
  }
if (rw->max>1)
  {
  GList *l;

  l=u->msgs;
  if (l!=NULL)
	{
	m=(msg_struct *)(l->data);
	free_msgstruct(m);
	u->msgs=g_list_remove(u->msgs,(gpointer)m);
	u->num_msgs--;
	rw->max--;
	readwin_update(rw);
	}
  else
	{
	puts("rw->u->msgs is NULL! (1)");
	}
  }
else
  {
  gtk_widget_set_sensitive(rw->but_next,FALSE);
  }

}


void readwin_close(readwin_struct *rw)
{
userstruct *u;
msg_struct *m;

/* we've already destroyed the window, so we don't do it now */

u=rw->u;
u->readwin=NULL;

/* free current msgstruct buffer */
if (u->msgs!=NULL)
  {
  m=(msg_struct *)(u->msgs->data);
  free_msgstruct(m);
  u->msgs=g_list_remove(u->msgs,(gpointer)m);
  u->num_msgs--;
  }
else
  {
  puts("rw->u->msgs is NULL! (3)");
  }

if (u->num_msgs==0)
  {
  remove_icon_from_queue(u,IQ_MESSAGE);
  }

g_free(rw);
}


gint readwin_delete_event(GtkWidget *widget, GdkEvent *event, gpointer data)
{
gtk_widget_destroy(widget);
return(TRUE);
}

void readwin_add(readwin_struct *rw)
{
rw->max++;
gtk_widget_set_sensitive(rw->but_next,TRUE);
}

void readwin_reply(readwin_struct *rw)
{
GList *l;

l=rw->u->msgs;
if (l==NULL) return;

quote((msg_struct *)(l->data));
}


void readwin_forward(readwin_struct *rw)
{
GList *l;

l=rw->u->msgs;
if (l==NULL) return;

forward_quote((msg_struct *)(l->data));
}


void quote(msg_struct *m)
{
char *msg=NULL, *qmsg=NULL;
signed int pri=0;
sendwin_struct *sn;

if (m==NULL) return;

/*sscanf(m->pri,"%d",&pri);*/ /*SEGFAULTS*/

/* FIXME: this quoting code sucks */
if (m->msg!=NULL) msg=g_strconcat(" > ",m->msg,NULL);
		     else msg=g_strdup(m->msg);

qmsg=strrepl(msg,"\n","\n > ");

sn=create_sendwin(m->fromid, m->fromnick, m->thread, m->subj, NULL, pri);

if ((qmsg!=NULL) && (gettoggle("noquote")==FALSE))
  {
  GdkFont *boldfont;
  GdkFont *normfont;

  gtk_text_freeze(GTK_TEXT(sn->text));
  boldfont = gdk_font_load ("-adobe-courier-bold-r-normal--*-120-*-*-*-*-*-*");
  normfont = gdk_font_load ("-adobe-helvetica-medium-r-normal--*-120-*-*-*-*-*-*");

  gtk_text_insert(GTK_TEXT(sn->text), normfont, NULL, NULL, "\n\n", -1);
  gtk_text_insert(GTK_TEXT(sn->text), boldfont, NULL, NULL, qmsg, -1);
  gtk_text_set_point(GTK_TEXT(sn->text), gtk_text_get_length(GTK_TEXT(sn->text)));
  gtk_text_insert(GTK_TEXT(sn->text), normfont, NULL, NULL, "\n\n", -1);

  gdk_font_unref(boldfont);
  gdk_font_unref(normfont);

  gtk_text_thaw(GTK_TEXT(sn->text));
  }

g_free(qmsg);
g_free(msg);
}


void forward_quote(msg_struct *m)
{
signed int pri=0;
sendwin_struct *sn;
char *buf=NULL;
char *subj=NULL;
GdkFont *boldfont;
GdkFont *normfont;

if (m==NULL) return;

/*sscanf(m->pri,"%d",&pri);*/ /*SEGFAULTS*/

if (m->fromid!=NULL)
  {
  buf=strgrow(buf,"\nFrom: ",1,0);
  buf=strgrow(buf,m->fromid,1,0);
  if (m->fromnick!=NULL)
	{
	buf=strgrow(buf," (",1,0);
	buf=strgrow(buf,m->fromnick,1,0);
	buf=strgrow(buf,")",1,0);
	}
  }
if (m->thread!=NULL)
  {
  buf=strgrow(buf,"\nThread: ",1,0);
  buf=strgrow(buf,m->thread,1,0);
  }
if (m->subj!=NULL)
  {
  buf=strgrow(buf,"\nSubject: ",1,0);
  buf=strgrow(buf,m->subj,1,0);
  }
if (m->pri!=NULL)
  {
  buf=strgrow(buf,"\nPriority: ",1,0);
  buf=strgrow(buf,m->pri,1,0);
  }
if (m->date!=NULL)
  {
  buf=strgrow(buf,"\nDate: ",1,0);
  buf=strgrow(buf,m->date,1,0);
  }

subj=strgrow(subj,m->subj,1,0);
subj=strgrow(subj," (fwd)",1,0);

sn=create_sendwin(NULL, NULL, m->thread, subj, NULL, pri);

g_free(subj);

gtk_text_freeze(GTK_TEXT(sn->text));
boldfont = gdk_font_load ("-adobe-courier-bold-r-normal--*-120-*-*-*-*-*-*");
normfont = gdk_font_load ("-adobe-helvetica-medium-r-normal--*-120-*-*-*-*-*-*");

gtk_text_insert(GTK_TEXT(sn->text), normfont, NULL, NULL, "\n\n", -1);
gtk_text_insert(GTK_TEXT(sn->text), boldfont, NULL, NULL, "--- Forwarded message follows ------", -1);
gtk_text_insert(GTK_TEXT(sn->text), normfont, NULL, NULL, buf, -1);
gtk_text_insert(GTK_TEXT(sn->text), boldfont, NULL, NULL, "------------------------------------\n", -1);
gtk_text_insert(GTK_TEXT(sn->text), boldfont, NULL, NULL, m->msg, -1);
gtk_text_insert(GTK_TEXT(sn->text), boldfont, NULL, NULL, "\n------------------------------------", -1);
gtk_text_insert(GTK_TEXT(sn->text), normfont, NULL, NULL, "\n\n", -1);
gtk_text_set_point(GTK_TEXT(sn->text), 0);

gdk_font_unref(boldfont);
gdk_font_unref(normfont);

gtk_text_thaw(GTK_TEXT(sn->text));

g_free(buf);
}
