package org.jabber;

import java.io.*;
import java.net.*;

import org.jabber.packet.pipeline.*;

/**
 * A <code>PipelineBuilder</code> follows the builder pattern from the GoF.
 * It creates an connects a packet pipeline for the given data transport.
 *
 * @author Michael T. Nygard
 * @version 1.0
 */
public class PipelineBuilder {
/** 
 * Build a packet pipeline appropriate to sending packets in and out
 * via a Reader and Writer.
 *
 * @param in The reader from which to read packets.
 * @parma out The writer to which packets send.
 * @return A connected pipeline, ready to send and receive packets.
 */
public static Pipeline constructOn(InputStream in, OutputStream out) {
	// Tail first
	Pipeline sender = new OutputStreamSender(out);
	Pipeline receiver = new InputStreamReceiver(in);
	Pipeline splitter = new PipeSplitter(sender, receiver);
	Pipeline parse = new PacketParser(splitter);
	return new PacketPreparer(parse);
}
/** 
 * Build a packet pipeline appropriate to sending packets over
 * a socket connection.
 *
 * @param sock The socket connection of interest
 * @return A connected pipeline, ready to send and receive packets.
 */
public static Pipeline constructOn(Socket sock) {
	// Tail first
	// Build the packet pipeline, back to front
	Pipeline tail = new SocketSender(sock);
	Pipeline parse = new PacketParser(tail);
	return new PacketPreparer(parse);
}
}