package org.jabber.client;

import java.io.*;
import java.net.*;

import org.jabber.*;

public class Main {
	private String host = null;
	private Integer port = null;
	private Channel channel = null;
	private MainFrame mainFrame = null;

	class ArgumentException extends Exception {
	public ArgumentException(String s) {
	    super(s);
	}
	}

	class WindowCloser extends java.awt.event.WindowAdapter {
	public void windowClosing(java.awt.event.WindowEvent evt) {
	    exit();
	}
	}

	public Main() {
	}
	protected Channel createChannel() {
	Channel chan = new Channel();

	if(getJabberHost() != null) {
	    chan.setTransportHost(getJabberHost());
	}

	if(getJabberPort() != null) {
	    chan.setTransportPort(getJabberPort().intValue());
	}

	return chan;
	}
	protected MainFrame createMainFrame() {
	MainFrame mf = new MainFrame();

	mf.setChannel(getChannel());

	mf.addWindowListener(new WindowCloser());

	return mf;
	}
	protected void exit() {
	System.exit(0);
	}
	public Channel getChannel() {
	if(channel == null) {
	    channel = createChannel();
	}

	return channel;
	}
	public String getJabberHost() {
	return this.host;
	}
	public Integer getJabberPort() {
	return this.port;
	}
	//
	// Aggregate objects (lazily initialized)
	//
	public MainFrame getMainFrame() {
	if(mainFrame == null) {
	    mainFrame = createMainFrame();
	}

	return mainFrame;
	}
	//
	// Command-line interface
	//
	public static void main(String[] args) {
	Main m = new Main();

	try {
	    m.parseArguments(args);
	} catch(ArgumentException arge) {
	    System.err.println(arge.getMessage());
	    System.err.println("\nUsage: " + usage());
	    System.exit(-1);
	} catch(IOException ioe) {
	    System.err.println(ioe);
	    System.exit(-2);
	}

	m.start();
	}
	public void parseArguments(String[] args) 
	throws ArgumentException, UnknownHostException 
	{
	for(int i = 0; i < args.length; i++) {
	    if("-h".equals(args[i])) {
		setJabberHost(args[++i]);
	    } else if("-p".equals(args[i])) {
		setJabberPort(new Integer(args[++i]));
	    } else {
		throw new ArgumentException("Unknown argument: " + args[i]);
	    }
	}
	}
	//
	// Properties
	//
	public void setJabberHost(String host) {
	setJabberHost(host);
	}
	public void setJabberPort(int port) {
	this.port = new Integer(port);
	}
	public void setJabberPort(Integer port) {
	this.port = port;
	}
	//
	// Lifecycle methods
	//
	protected void start() {
	MainFrame mf = getMainFrame();
	
	mf.setVisible(true);
	}
	public static String usage() {
	return "java org.jabber.client.Main [-h jabber_host]";
	}
}