package org.jabber.packet;

import java.io.*;
import java.util.*;

import org.jabber.*;
import org.jabber.element.*;

public class ProtocolBuilder implements PacketVisitor {
	private Element rootElement;
	
	class ElementWritingVisitor implements ElementVisitor {
		private StringBuffer buffer = new StringBuffer();
		private int indent;
		
		public String toString() {
			return buffer.toString();
		}
		
		public void visit(Attribute attr) {
			buffer.append(attr.getName());
			buffer.append("=\"");
			buffer.append(attr.getValue());
			buffer.append("\"");
		}
		
		public void visit(TextElement text) {
			buffer.append("\n");
			indent();
			bufferStartTag(text);
			buffer.append(text.getText());
			bufferEndTag(text);
		}
		
		public void visit(Element elt) {
			Enumeration e;
			indent();
			bufferStartTag(elt);
			if(!elt.isEmpty()) {
				bufferChildren(elt);
			}
			buffer.append("\n");
			bufferEndTag(elt);
		}

		protected void bufferStartTag(Element elt) {
			buffer.append("<" + elt.getTagName());
			for (Enumeration e = elt.attributes(); e.hasMoreElements();) {
				Attribute attr = (Attribute) e.nextElement();
				attr.accept(this);
			}
			if(elt.isEmpty()) {
				buffer.append("/>");
			} else {
				buffer.append(">");
			}
		}

		protected void bufferChildren(Element elt) {
			indent(1);
			Enumeration e = elt.children();
			while (e.hasMoreElements()) {
				Element child = (Element) e.nextElement();
				child.accept(this);
			}
			indent(-1);
		}
		
		protected void bufferEndTag(Element elt) {
			if(elt.isEmpty()) {
				return;
			}
			buffer.append("</");
			buffer.append(elt.getTagName());
			buffer.append(">");
		}
		
		protected void indent() {
			for (int i = 0; i < indent; i++) {
				buffer.append(' ');
			}
		}
		
		protected void indent(int incr) {
			indent += incr;
		}
	}
public Element getRootElement() {
	return rootElement;
}
protected void setRootElement(Element elt) {
	rootElement = elt;
}
public RawPacket unparse(Packet p) {
	RawPacket raw = null;

	p.accept(this);

	if(getRootElement() != null) {
		raw = new RawPacket();
		
		ElementWritingVisitor v = new ElementWritingVisitor();
		getRootElement().accept(v);

		raw.setRawBytes(v.toString().getBytes());
	}
	
	return raw;
}
public void visit(ConnectionPacket conn) {
	// This shouldn't get called
	setRootElement(null);
}
public void visit(LoginPacket login) {
	Element root = new Element("login");

	root.addTextElement("name", login.getName());
	root.addTextElement("password", login.getPassword());
	root.addTextElement("nick", login.getNick());

	setRootElement(root);
}
public void visit(MessagePacket message) {
	Element root = new Element("message");

	for(Enumeration e = message.getTo(); e.hasMoreElements(); ) {
		root.addTextElement("to", (String)e.nextElement());
	}

	root.addTextElement("from", message.getFrom());
	root.addTextElement("priority", message.getPriority());
	root.addTextElement("say", message.getSay());
	root.addTextElement("subject", message.getSubject());
	root.addTextElement("thread", message.getThread());

	setRootElement(root);
}
public void visit(RawPacket raw) {
	// This shouldn't get called
	setRootElement(null);
}
public void visit(RosterPacket roster) {
}
public void visit(StatusPacket status) {
	Element root = new Element("status");

	root.addTextElement("say", status.getSay());
	root.addTextElement("priority", status.getPriority());
	root.addTextElement("from", status.getFrom());
	root.addTextElement("icon", status.getIconName());

	setRootElement(root);
}
}