package org.jabber.packet;

import org.jabber.*;

import java.io.*;
import java.util.*;

/**
 * A <code>RawPacket</code> is what comes off of the wire.
 * It has not been parsed into elements and attributes yet.
 * If sent from the head of the pipeline, raw packets do
 * not get parsed or unparsed.
 * 
 * @author Michael T. Nygard
 * @version 1.0
 */
public class RawPacket extends Packet {
	private byte[] rawBytes;
	private InputStream inputSource;
public RawPacket() {
	super();
}
public RawPacket(byte[] bytes) {
	super();
	rawBytes = bytes;
}
public RawPacket(String str) {
	super();
	rawBytes = str.getBytes();
}
public void accept(PacketVisitor v) {
	v.visit(this);
}
public InputStream getInputSource() {
	return this.inputSource;
}
protected String getPacketType() {
	return "raw";
}
public InputStream getRawAsInputStream() {
	return new ByteArrayInputStream(rawBytes);
}
public byte[] getRawBytes() {
	return rawBytes;
}
public boolean isSelfFormatting() {
	return true;
}
public void setInputSource(InputStream in) {
	this.inputSource = in;
}
public void setRawBytes(byte[] buffer) {
	this.rawBytes = buffer;
}
}