package org.jabber.packet.pipeline.test;

import java.io.*;

import junit.framework.*;

import org.jabber.packet.*;
import org.jabber.packet.pipeline.*;

public class FilteredPipelineTest extends TestCase {
	class PipelineCascades extends Pipeline {
		private Packet returnPacket = null;
		private String lastMethod = null;

		public void setReturnPacket(Packet p)			{ returnPacket = p;		}
		public String getLastMethod() 					{ return lastMethod;	}
		public void close() throws IOException			{ lastMethod = "close";	}
		public void send(Packet p) throws IOException	{ lastMethod = "send";	}
		public Packet receive() throws IOException		{ lastMethod = "receive"; return returnPacket; }
	}

	class PipelineExceptions extends Pipeline {
		public void close() throws IOException {
			throw new IOException("close");
		}

		public void send(Packet p) throws IOException {
			throw new IOException("send");
		}

		public Packet receive() throws IOException {
			throw new IOException("receive");
		}
	}
/**
 * FilteredPipelineTest constructor comment.
 * @param name java.lang.String
 */
public FilteredPipelineTest(String name) {
	super(name);
}
public static void main(java.lang.String[] args) {
	junit.textui.TestRunner.run(suite());
}
public static Test suite() {
	return new TestSuite(FilteredPipelineTest.class);
}
public void testCloseCascade() {
	PipelineCascades p = new PipelineCascades();
	
	FilteredPipeline testPipe = new FilteredPipeline(p);

	try {
		testPipe.close();

		assert(p.getLastMethod() != null);
		assert(p.getLastMethod().equals("close"));
	} catch(IOException ioe) {
		fail("Unexpected IOException: " + ioe);
	}
}
public void testCloseException() {
	PipelineExceptions p = new PipelineExceptions();
	FilteredPipeline testPipe = new FilteredPipeline(p);

	try {
		testPipe.close();

		fail("Exception not passed up!");
	} catch(IOException ioe) {
		// OK
	}
}
public void testReceiveCascade() {
	PipelineCascades p = new PipelineCascades();
	
	FilteredPipeline testPipe = new FilteredPipeline(p);

	try {
		testPipe.receive();

		assert(p.getLastMethod() != null);
		assert(p.getLastMethod().equals("receive"));

		RawPacket rawP = new RawPacket();
		
		p.setReturnPacket(rawP);

		Packet retVal = testPipe.receive();

		assert(retVal == rawP);
	} catch(IOException ioe) {
		fail("Unexpected IOException: " + ioe);
	}
}
public void testReceiveException() {
	PipelineExceptions p = new PipelineExceptions();
	FilteredPipeline testPipe = new FilteredPipeline(p);

	try {
		testPipe.receive();

		fail("Exception not passed up!");
	} catch(IOException ioe) {
		// OK
	}
}
public void testReceiveFaithful() {
	PipelineCascades p = new PipelineCascades();
	
	FilteredPipeline testPipe = new FilteredPipeline(p);

	try {
		RawPacket rawP = new RawPacket();
		
		p.setReturnPacket(rawP);

		Packet retVal = testPipe.receive();

		assert(retVal == rawP);
	} catch(IOException ioe) {
		fail("Unexpected IOException: " + ioe);
	}
}
public void testSendCascade() {
	PipelineCascades p = new PipelineCascades();
	
	FilteredPipeline testPipe = new FilteredPipeline(p);

	try {
		testPipe.send(new RawPacket());

		assert(p.getLastMethod() != null);
		assert(p.getLastMethod().equals("send"));

		testPipe.send(null);
	} catch(IOException ioe) {
		fail("Unexpected IOException: " + ioe);
	}
}
public void testSendException() {
	PipelineExceptions p = new PipelineExceptions();
	FilteredPipeline testPipe = new FilteredPipeline(p);

	try {
		testPipe.send(new RawPacket());

		fail("Exception not passed up!");
	} catch(IOException ioe) {
		// OK
	}
}
public void testSendNull() {
	PipelineCascades p = new PipelineCascades();
	
	FilteredPipeline testPipe = new FilteredPipeline(p);

	try {
		testPipe.send(null);
	} catch(IOException ioe) {
		fail("Unexpected IOException: " + ioe);
	}
}
}