package org.jabber.packet.pipeline.test;

import java.io.*;

import junit.framework.*;

import org.jabber.packet.*;
import org.jabber.packet.pipeline.*;

public class PacketPreparerTest extends TestCase {
	class TestPacket extends Packet {
		private String lastMethod;
		public String getLastMethod()		{ return lastMethod;	}
		public void accept(PacketVisitor v)	{ }
		public String getPacketType()		{ return "TEST_PACKET"; }
		public void willSend()				{ lastMethod = "willSend"; }
		public void didReceive()			{ lastMethod = "didReceive"; }
	}

	class PipelineStub extends Pipeline {
		private Packet returnPacket;
		public PipelineStub()				{ }
		public PipelineStub(Packet p)		{ returnPacket = p; }
		public void send(Packet p)			{ }
		public Packet receive()				{ return returnPacket; }
	}
/**
 * PacketPreparerTest constructor comment.
 * @param name java.lang.String
 */
public PacketPreparerTest(String name) {
	super(name);
}
protected PacketPreparer createPreparer(boolean returnsNull) {
	Packet stubPacket;
	if (returnsNull) {
		stubPacket = null;
	} else {
		stubPacket = new TestPacket();
	}
	return new PacketPreparer(new PipelineStub(stubPacket));
}
public static void main(java.lang.String[] args) {
	junit.textui.TestRunner.run(suite());
}
public static Test suite() {
	return new TestSuite(PacketPreparerTest.class);
}
public void testDidReceive() throws IOException {
	PacketPreparer p = createPreparer(false);

	Packet packet = p.receive();

	assert(packet != null);
	assert(packet instanceof TestPacket);

	TestPacket test = (TestPacket)packet;
	
	assert( test.getLastMethod() != null );
	assert( test.getLastMethod().equals("didReceive") );
}
public void testDidReceiveNull() throws IOException {
	PacketPreparer p = createPreparer(true);
	
	p.receive();
}
public void testWillSend() throws IOException {
	PacketPreparer p = createPreparer(true);

	TestPacket tp = new TestPacket();

	p.send(tp);

	assert(tp.getLastMethod() != null && tp.getLastMethod().equals("willSend"));
}
public void testWillSendNull() throws IOException {
	PacketPreparer p = createPreparer(true);

	p.send(null);
}
}