package org.jabber.packet.pipeline.test;

import java.io.*;

import junit.framework.*;

import org.jabber.packet.*;
import org.jabber.packet.pipeline.*;

public class PipeSplitterTest extends TestCase {
	abstract class TestPipeline extends Pipeline {
		private String lastMethod;
		public String getLastMethod() {
			return lastMethod;
		}
		public void close() throws IOException {
			lastMethod = "close";
		}
	}
	class TestSender extends TestPipeline {
		public Packet receive() throws IOException {
			throw new IOException("Sender should not be asked to receive");
		}
		public void send(Packet p) throws IOException {
			lastMethod = "send";
		}
	}
	class TestReceiver extends TestPipeline {
		private Packet returnPacket;
		public TestReceiver(Packet p) {
			returnPacket = p;
		}
		public Packet receive() throws IOException {
			lastMethod = "receive";
			return returnPacket;
		}
		public void send(Packet p) throws IOException {
			throw new IOException("Receive should not be asked to send");
		}
	}
/**
 * PipeSplitterTest constructor comment.
 * @param name java.lang.String
 */
public PipeSplitterTest(String name) {
	super(name);
}
protected PipeSplitter createSplitter() {
	return new PipeSplitter(new TestSender(), new TestReceiver(new RawPacket("<say>foo</say>")));
}
public static void main(java.lang.String[] args) {
	junit.textui.TestRunner.run(suite());
}
public static Test suite() {
	return new TestSuite(PipeSplitterTest.class);
}
public void testClose() throws IOException {
	PipeSplitter splitter = createSplitter();

	splitter.close();

	assert( splitter.getSender() instanceof TestSender );
	assert( ((TestSender)splitter.getSender()).getLastMethod().equals("close") );

	assert( splitter.getReceiver() instanceof TestReceiver );
	assert( ((TestReceiver)splitter.getReceiver()).getLastMethod().equals("close") );
}
public void testReceive() throws IOException {
	PipeSplitter splitter = createSplitter();

	Packet p = splitter.receive();

	assert(splitter.getReceiver() instanceof TestReceiver);
	assert( ((TestReceiver)splitter.getReceiver()).getLastMethod().equals("receive"));

	assert(splitter.getSender() instanceof TestSender);
	assert( ((TestSender)splitter.getSender()).getLastMethod() == null);
}
public void testSend() throws IOException {
	PipeSplitter splitter = createSplitter();

	splitter.send(new RawPacket("<say>foo</say>"));

	assert(splitter.getReceiver() instanceof TestReceiver);
	assert( ((TestReceiver)splitter.getReceiver()).getLastMethod() == null);

	assert(splitter.getSender() instanceof TestSender);
	assert( ((TestSender)splitter.getSender()).getLastMethod().equals("send"));
}
}