package org.jabber.packet.pipeline.test;

import java.io.*;
import junit.framework.*;
import org.jabber.packet.*;
import org.jabber.packet.pipeline.*;
/**
 * This type was created in VisualAge.
 */
public class PipelineTailTest extends TestCase {
	class TestTail extends PipelineTail {
		private RawPacket returnPacket = null;
		private String lastMethod = null;
		
		public String getLastMethod() {
			return lastMethod;
		}
		
		public void setReturnPacket(RawPacket p) {
			returnPacket = p;
		}

		public RawPacket receiveRaw() throws IOException {
			lastMethod = "receiveRaw";
			return returnPacket;
		}

		public void sendRaw(RawPacket raw) throws IOException {
			lastMethod = "sendRaw";
		}
	}
	class PipelineExceptions extends PipelineTail {
		public void sendRaw(RawPacket p) throws IOException {
			throw new IOException("send");
		}

		public RawPacket receiveRaw() throws IOException {
			throw new IOException("receive");
		}
	}
/**
 * PipelineTailTest constructor comment.
 * @param name java.lang.String
 */
public PipelineTailTest(String name) {
	super(name);
}
public static void main(java.lang.String[] args) {
	junit.textui.TestRunner.run(suite());
}
public static Test suite() {
	return new TestSuite(PipelineTailTest.class);
}
public void testOnlySendsRaw() {
	TestTail t = new TestTail();
	
	try {
		t.send(new StatusPacket("from", "say", new Integer(5), "iconName"));
		assert(false);
	} catch(IOException ioe) {
		// Ok
	}

	try {
		t.send(new LoginPacket("username", "nick"));
		assert(false);
	} catch(IOException ioe) {
		// Ok
	}
}
public void testReceivesRaw() {
	TestTail t = new TestTail();

	try {
		t.setReturnPacket(new RawPacket("<say>foo</say>"));
		Packet p = t.receive();

		assert(p instanceof RawPacket);
	} catch(IOException ioe) {
		fail("Unexpected IOException: " + ioe);
	}
}
public void testSendException() {
	PipelineExceptions t = new PipelineExceptions();

	try {
		t.send(new RawPacket());

		fail("Exception not passed up!");
	} catch(IOException ioe) {
		// OK
	}
}
public void testSendsRaw() {
	TestTail t = new TestTail();

	try {
		t.send(new RawPacket("<say>foo</say>"));
		assert(t.getLastMethod().equals("sendRaw"));
	} catch(IOException ioe) {
		fail("Unexpected IOException: " + ioe);
	}
}
}