package org.jabber.packet.test;

import java.io.*;

import junit.framework.*;

import org.jabber.*;
import org.jabber.packet.*;
import org.jabber.packet.pipeline.*;

/**
 * Unit tests for the StatusPacket.
 *
 * @author Michael T. Nygard
 *
 */
public class StatusPacketTest extends PacketTest {
	private Pipeline pipe;
public StatusPacketTest(String name) {
	super(name);
}
public static void main(String[] args) throws IOException {
	junit.textui.TestRunner.run(suite());
}
public void setUp() {
	pipe = PipelineBuilder.constructOn(System.in, System.out);
}
public static Test suite() {
	return new TestSuite(StatusPacketTest.class);
}
public void testConstructor() {
	StatusPacket packet = new StatusPacket("from", "say", new Integer(5), "icon");
	
	assert(packet.getFrom().equals("from"));
	assert(packet.getSay().equals("say"));
	assert(packet.getPriority().intValue() == 5);
	assert(packet.getIconName().equals("icon"));
}
public void testFullPacket() {
	StatusPacket packet = new StatusPacket("FromTest", "SayTest", new Integer(9), "IconTest");

	checkRepresentation(packet, "<status>\n <say>SayTest</say>\n <priority>9</priority>\n <from>FromTest</from>\n <icon>IconTest</icon>\n</status>");
}
public void testIcon() {
	StatusPacket packet = new StatusPacket(null, null, null, "IconTest");

	checkRepresentation(packet, "<status>\n <icon>IconTest</icon>\n</status>");
}
public void testPacketType() {
	StatusPacket packet = StatusPacket.prototype();
	assert(PacketTypes.STATUS.equals(packet.getPacketType()));
}
public void testPriority() {
	StatusPacket packet = new StatusPacket(null, null, new Integer(5), null);

	checkRepresentation(packet, "<status>\n <priority>5</priority>\n</status>");
}
public void testPrototype() {
	StatusPacket packet = StatusPacket.prototype();

	assert(packet.getFrom() == null);
	assert(packet.getIconName() == null);
	assert(packet.getPriority() == null);
	assert(packet.getSay() == null);
}
public void testSay() {
	StatusPacket packet = new StatusPacket(null, "SayTest", null, null);

	checkRepresentation(packet, "<status>\n <say>SayTest</say>\n</status>");
}
public void testVisitor() {
	StatusPacket packet = StatusPacket.prototype();

	PacketVisitor v = new PacketVisitor() {
		public void visit(ConnectionPacket p) {
			fail("Why am I visiting a ConnectionPacket?");
		}
		public void visit(LoginPacket p) {
			fail("Why am I visiting a LoginPacket?");
		}
		public void visit(MessagePacket p) {
			fail("Why am I visiting a LoginPacket?");
		}
		public void visit(RawPacket p) {
			fail("Why am I visiting a LoginPacket?");
		}
		public void visit(RosterPacket p) {
			fail("Why am I visiting a LoginPacket?");
		}
		public void visit(StatusPacket p) {
		}
	};

	packet.accept(v);
}
}