#
# JabberLib
#   by Kerem 'Waster_' HADIMLI
#
# Contact me at waster@iname.com
#

#
# Variables used in JabberLib :
#	JabberLib(temp)                         : Used for parsing messages, statuses...
#
#	JabberLib(server,protocol)              : Protocol of the server
#	JabberLib(server,version)               : Version of the server
#
#	JabberLib(roster,groups)                : Names of the groups currently in roster
#	JabberLib(roster,groups,$groupname)     : Users in $groupname
#
#	JabberLib(roster,users)                 : Names of the users currently in roster
#	JabberLib(roster,users,$username)       : Avaible nicks of $username
#	JabberLib(roster,users,$username,$nick) : Properties of $nick. Properties consists of: type, message, priority, and icon
#
#	JabberLib(status,type)                  : Status:type for user
#	JabberLib(status,say)                   : Status:message for user
#	JabberLib(status,pri)                   : Status:priority for user
#	JabberLib(status,icon)                  : Status:icon for user
#
#	JabberLib(sck)                          : SocketName
#	JabberLib(sckstats)                     : Socket status, "on" or "off"
#
#	JabberLib(file)                         : Threads of files being transferred.
#	JabberLib(file,$thread,curpack)         : Most recent packet number
#       JabberLib(file,$thread,name)            : Filename
#       JabberLib(file,$thread,length)          : Length of file
#       JabberLib(file,$thread,inout)           : "out" if sending, "in" if getting
#       JabberLib(file,$thread,ttype)           : TransferType
#

set JabberLib(server,protocol) ""
set JabberLib(server,version) ""

set JabberLib(roster,groups) ""
set JabberLib(roster,users) ""

set JabberLib(status,type) ""
set JabberLib(status,say)  ""
set JabberLib(status,pri)  ""
set JabberLib(status,icon) ""

set JabberLib(sck) ""
set JabberLib(sckstats) "off"

set JabberLib(file) ""

#
# Loading XML:parser
#
source "[file dirname [info script]]/parser.tcl"

#############################################
# Here starts JabberLib_2_Client procedures #
#############################################

#
# The main procedure to parse incoming text
#
proc JabberLib:parse:main {tag vars between} {
global JabberLib

set tag [string tolower $tag]

  switch -- $tag {
	  ?xml {}
	jabber {
		foreach {attr value} $vars {
			switch -- $attr {
				protocol {set JabberLib(server,protocol) $value}
				version  {set JabberLib(server,version)  $value}
			}
		}
		client:jabber:server $JabberLib(server,protocol) $JabberLib(server,version)
	       }
       message {
		set type "normal"
		foreach {attr value} $vars {
			switch -- $attr {
				type {set type $value}
			}
		}
		if {$type=="error"} {
				     set JabberLib(temp) {"" "" "" ""}
				     XML:parse $between "JabberLib:parse:errmsg"
				     eval client:msg:error $JabberLib(temp)
				     set JabberLib(temp) ""} \
		  elseif {$type=="normal"} {
					    set JabberLib(temp) {"" "" "" "" "" ""}
					    XML:parse $between "JabberLib:parse:msg"
					    eval client:msg:normal $JabberLib(temp)
					    set JabberLib(temp) ""}
	       }
	roster {XML:parse $between "JabberLib:parse:roster"}
	status {
		set JabberLib(temp) {"" "" "" "" "" ""}

		foreach {attr value} $vars {
			switch -- $attr {
				type {set JabberLib(temp) [lreplace $JabberLib(temp) 2 2 $value]}
			}
		}

		XML:parse $between "JabberLib:parse:status"
		set user [lindex $JabberLib(temp) 0]
		set nick [lindex $JabberLib(temp) 1]
		set prop [lrange $JabberLib(temp) 2 5]

		if {$user=="" || $nick=="" || [lsearch $JabberLib(roster,users) $user]==-1} {set JabberLib(temp) ""; return}

		if {[lsearch $JabberLib(roster,users,$user) $nick]==-1} {lappend JabberLib(roster,users,$user) $nick
									 set JabberLib(roster,users,$user,$nick) {"offline" "" "0" "normal"}
									}
		if {[lindex $prop 0]==""} {set prop [lreplace $prop 0 0 [lindex $JabberLib(roster,users,$user,$nick) 0]]}
		if {[lindex $prop 2]==""} {set prop [lreplace $prop 2 2 [lindex $JabberLib(roster,users,$user,$nick) 2]]}
		if {[lindex $prop 3]==""} {set prop [lreplace $prop 3 3 [lindex $JabberLib(roster,users,$user,$nick) 3]]}

		set JabberLib(roster,users,$user,$nick) $prop
		client:roster:nick $user $nick $prop
		set JabberLib(temp) ""
	       }
  }
}

#
# The procedure to parse Error Messages
#
proc JabberLib:parse:errmsg {tag vars between} {
global JabberLib
set tag [string tolower $tag]
set between [XML:decrypt $between]

  switch -- $tag {
	from {set JabberLib(temp) [lreplace $JabberLib(temp) 0 0 $between]}
     subject {set JabberLib(temp) [lreplace $JabberLib(temp) 1 1 $between]}
	 say {set JabberLib(temp) [lreplace $JabberLib(temp) 2 2 $between]}
	 ext {set JabberLib(temp) [lreplace $JabberLib(temp) 3 3 $between]}
  }
}

#
# The procedure to parse Normal Messages
#
proc JabberLib:parse:msg {tag vars between} {
global JabberLib
set tag [string tolower $tag]
set between [XML:decrypt $between]

  switch -- $tag {
	from {
	      foreach {attr value} $vars {
		switch -- $attr {
			nick {set JabberLib(temp) [lreplace $JabberLib(temp) 1 1 $value]}
		}
	      }
	      set JabberLib(temp) [lreplace $JabberLib(temp) 0 0 $between]
	     }
     subject {set JabberLib(temp) [lreplace $JabberLib(temp) 2 2 $between]}
      thread {set JabberLib(temp) [lreplace $JabberLib(temp) 3 3 $between]}
    priority {set JabberLib(temp) [lreplace $JabberLib(temp) 4 4 $between]}
	 say {set JabberLib(temp) [lreplace $JabberLib(temp) 5 5 $between]}
  }
}

#
# The procedure to parse Status Messages
#
proc JabberLib:parse:status {tag vars between} {
global JabberLib
set tag [string tolower $tag]
set between [XML:decrypt $between]

  switch -- $tag {
	from {
	      foreach {attr value} $vars {
			switch -- $attr {
				nick {set JabberLib(temp) [lreplace $JabberLib(temp) 1 1 $value]}
			}
	      }
	      set JabberLib(temp) [lreplace $JabberLib(temp) 0 0 $between]
	     }
	 say {
	      set JabberLib(temp) [lreplace $JabberLib(temp) 3 3 $between]
	     }
    priority {set JabberLib(temp) [lreplace $JabberLib(temp) 4 4 $between]}
	icon {set JabberLib(temp) [lreplace $JabberLib(temp) 5 5 $between]}
  }
}

#
# The procedure to parse Roster Messages
#
proc JabberLib:parse:roster {tag vars between} {
global JabberLib
set tag [string tolower $tag]
set between [XML:decrypt $between]

  switch -- $tag {
	group {
	       set groupname ""
	       foreach {attr value} $vars {
			switch -- $attr {
				name {set groupname $value}
			}
	       }
	       set username $between
	       if {[lsearch $JabberLib(roster,users) $username]==-1} {lappend JabberLib(roster,users) $username
								      set JabberLib(roster,users,$username) ""
								     }
	       if {[lsearch $JabberLib(roster,groups) $groupname]==-1} {lappend JabberLib(roster,groups) $groupname
									set JabberLib(roster,groups,$groupname) ""
									client:roster:group $groupname
								       }
	       if {$username==""} {JabberLib:outmsg "<roster><get group='$groupname'/></roster>"
				   return
				  }
	       if {[lsearch $JabberLib(roster,groups,$groupname) $username]==-1} {lappend JabberLib(roster,groups,$groupname) $username
										   client:roster:user $groupname $username
										 }
	      }
  }
}

#
# This procedure is for notifying client
# when server closes connection
#
proc JabberLib:EndOfParse {} {
  JabberLib:jabber:disconnect
  client:jabber:offline
}

#############################################
# Here starts Client_2_JabberLib procedures #
#############################################

#
# This procedure resends all Roster Info to client
#
proc JabberLib:roster:refresh {} {
global JabberLib

  foreach group $JabberLib(roster,groups) {
	client:roster:group $group                      ;#Here we send the groupname
	foreach user $JabberLib(roster,groups,$group) {
		client:roster:user $group $user         ;#Here we send users in the group
	}
  }

  foreach user $JabberLib(roster,users) {
	foreach nick $JabberLib(roster,users,$user) {
		set prop $JabberLib(roster,users,$user,$nick)
		client:roster:nick $user $nick $prop    ;#Here we send properties of the nick
	}
  }
}

#
# The procedure to connect to server
#
proc JabberLib:jabber:connect {server version user nick pass} {
global JabberLib

  if {[catch {set JabberLib(sck) [socket $server 5222]}]!=0} {return -1}

  set JabberLib(sckstats) "on"
  fconfigure $JabberLib(sck) -blocking 0

  set t1 {<?xml version="1.0"?>}
  set t2 "<jabber version=\"$version\" protocol=\"19990324\"><login>"
  if {$user!=""} {set t3 "<user>[XML:crypt $user]</user>"} else {set t3 ""}
  if {$nick!=""} {set t4 "<nick>[XML:crypt $nick]</nick>"} else {set t4 ""}
  if {$pass!=""} {set t5 "<pass>[XML:crypt $pass]</pass>"} else {set t5 ""}
  set t6 "</login>"

  JabberLib:outmsg "$t1$t2$t3$t4$t5$t6"

  fileevent $JabberLib(sck) readable {
	if {[catch {puts -nonewline $JabberLib(sck) " ";flush $JabberLib(sck)}]!=0} {
		JabberLib:jabber:disconnect
		client:jabber:offline
		return}
	if {[catch {
		   .debugwin.f0.text configure -state normal
		   .debugwin.f0.text insert end "\nServer: [set a [read $JabberLib(sck)]]"
		   .debugwin.f0.text see end
		   .debugwin.f0.text configure -state disabled
		   XML:parse $a "JabberLib:parse:main"
		}]!=0} {
		   JabberLib:jabber:disconnect
		   client:jabber:offline
		   return}
  }
  return 0
}

#
# The procedure to disconnect from server
#
proc JabberLib:jabber:disconnect {} {
global JabberLib

  set JabberLib(status,type) ""
  set JabberLib(status,say)  ""
  set JabberLib(status,pri)  ""
  set JabberLib(status,icon) ""

  foreach a $JabberLib(roster,groups) {
	catch {unset JabberLib(roster,groups,$a}
  }

  foreach a $JabberLib(roster,users) {
	foreach b $JabberLib(roster,users,$a) {
		catch {unset JabberLib(roster,users,$a,$b)}
	}
	catch {unset JabberLib(roster,users,$a)}
  }

  set JabberLib(roster,groups) ""
  set JabberLib(roster,users) ""

  JabberLib:outmsg "</jabber>"
  catch {close $JabberLib(sck)}
  set JabberLib(sckstats) "off"
}

#
# The procedure to send messages to server
#
proc JabberLib:outmsg {msg} {
global JabberLib
if {$JabberLib(sckstats)=="off"} {return}

  catch {
       .debugwin.f0.text configure -state normal
       .debugwin.f0.text insert end "\nClient: $msg"
       .debugwin.f0.text see end
       .debugwin.f0.text configure -state disabled

  	puts -nonewline $JabberLib(sck) $msg
	flush $JabberLib(sck)
  }
}

#
# This procedure changes and sends status
#
proc JabberLib:change:status {type msg pri icon} {
global JabberLib
if {$JabberLib(sckstats)=="off"} {return}

# MODIFIED FOR zABBER
if {$icon=="normal"} {set icon ""}
# /MODIFIED FOR zABBER

  if {($type=="" || $type==$JabberLib(status,type)) && $msg==$JabberLib(status,say) && ($pri=="" || $pri==$JabberLib(status,pri)) && ($icon=="" || $icon==$JabberLib(status,icon))} {return}

  if {$JabberLib(status,type)!=$type && $type!=""} {set JabberLib(status,type) $type; set t1 "<status type='$type'>"} \
	else {set t1 "<status>"}
  if {$JabberLib(status,say)!=$msg} {set JabberLib(status,say) $msg; set t2 "<say>[XML:crypt $msg]</say>"} \
	else {set t2 ""}
  if {$JabberLib(status,pri)!=$pri && $pri!=""} {set JabberLib(status,pri) $pri; set t3 "<priority>$pri</priority>"} \
	else {set t3 ""}
  if {$JabberLib(status,icon)!=$icon && $icon!=""} {set JabberLib(status,icon) $icon; set t4 "<icon>$icon</icon>"} \
	else {set t4 ""}
  set t5 "</status>"

  JabberLib:outmsg "$t1$t2$t3$t4$t5"
}

#
# This procedure returns properties of given nick
#
proc JabberLib:roster:getinfo {user nick} {
global JabberLib

  if {[lsearch $JabberLib(roster,users) $user]==-1} {return}
  if {[lsearch $JabberLib(roster,users,$user) $nick]==-1} {return}

  return $JabberLib(roster,users,$user,$nick)
}

#
# The procedure to add an empty group
#
proc JabberLib:roster:addgroup {group} {
global JabberLib

  if {[lsearch $JabberLib(roster,groups) $group]!=-1} {return}

  lappend JabberLib(roster,groups)        $group
  set     JabberLib(roster,groups,$group) ""
  JabberLib:outmsg "<roster><add group='[XML:crypt $group]'/></roster>"
}

#
# The procedure to add a user to a group
#
proc JabberLib:roster:adduser {group user} {
global JabberLib

  if {$user=="" || $group==""} {return}
  if {[lsearch $JabberLib(roster,groups) $group]==-1} {
						       lappend JabberLib(roster,groups) $group
						       set JabberLib(roster,groups,$group) ""
						      }
  if {[lsearch $JabberLib(roster,groups,$group) $user]!=-1} {return}

  lappend JabberLib(roster,groups,$group) $user
  if {[lsearch $JabberLib(roster,users) $user]==-1} {
						     lappend JabberLib(roster,users) $user
						     set JabberLib(roster,users,$user) ""
						    }

  JabberLib:outmsg "<roster><add group='[XML:crypt $group]'>[XML:crypt $user]</add></roster>"
}

#
# This procedure deletes user from group
#
proc JabberLib:roster:deluser {group user} {
global JabberLib

set userinall "-1"   ;#If the user exists in any other group...

  if {[set usernum [lsearch $JabberLib(roster,groups,$group) $user]]=="-1"} {return}
  set JabberLib(roster,groups,$group) [lreplace $JabberLib(roster,groups,$group) $usernum $usernum] ;#Delete user from group

  foreach a $JabberLib(roster,groups) {if {[lsearch $JabberLib(roster,groups,$a) $user]!=-1} {set userinall "1"}}
  if {$userinall=="-1"} {
	foreach b $JabberLib(roster,users,$user) {
		unset JabberLib(roster,users,$user,$b)
	}
	unset JabberLib(roster,users,$user) ;#Delete user from users
	set JabberLib(roster,users) [lreplace $JabberLib(roster,users) [set usernum [lsearch $JabberLib(roster,users) $user]] $usernum]
  }

  JabberLib:outmsg "<roster><del group=\"[XML:crypt $group]\">[XML:crypt $user]</del></roster>"
}

#
# This procedure deletes a group
#
proc JabberLib:roster:delgroup {group} {
global JabberLib

set userinall "-1"
set groupnum [lsearch $JabberLib(roster,groups) $group]

  if {$groupnum==-1} {return}

  foreach user $JabberLib(roster,groups,$group) {    #Here we delete the users in the group
	if {[set usernum [lsearch $JabberLib(roster,groups,$group) $user]]=="-1"} {break}
	foreach a $JabberLib(roster,groups) {if {[lsearch $JabberLib(roster,groups,$a) $user]!=-1} {set userinall "1"}}
	if {$userinall=="-1"} {
		foreach b $JabberLib(roster,users,$user) {
			unset JabberLib(roster,users,$user,$b)
		}
	unset JabberLib(roster,users,$user) ;#Delete user from users
	}

  }


  set JabberLib(roster,groups) [lreplace $JabberLib(roster,groups) $groupnum $groupnum]
  unset JabberLib(roster,groups,$group)

  JabberLib:outmsg "<roster><del group=\"[XML:crypt $group]\"/></roster>"
}

#
# This procedure sends a message
#
proc JabberLib:send:message {to nick sub thr pri mes} {
set out ""

  if {$to==""} {return}
  if {[llength $to]>1} {set nick ""} ;#Set $nick to "" if message is multiple..

  foreach a $to {
    if {$a==""} {continue}
    set msg "<message>"
    set out $out$msg
    if {$nick==""} {set msg "<to>[XML:crypt $a]</to>"; set out $out$msg}
    if {$nick!=""} {set msg "<to nick=\"[XML:crypt $nick]\">[XML:crypt $to]</to>"; set out $out$msg}
    if {$thr!=""} {set msg "<thread>[XML:crypt $thr]</thread>"; set out $out$msg}
    if {$pri!="0"} {set msg "<priority>$pri</priority>"; set out $out$msg}
    if {$sub!=""} {set msg "<subject>[XML:crypt $sub]</subject>"; set out $out$msg}
    if {$mes!=""} {set msg "<say>[XML:crypt $mes]</say>"; set out $out$msg}
    set msg "</message>"
    set out $out$msg
  }
  JabberLib:outmsg $out
}

#
# This procedure regets all roster from server
#
proc JabberLib:roster:reget {} {
global JabberLib

  foreach a $JabberLib(roster,groups) {
	catch {unset JabberLib(roster,groups,$a}
  }

  foreach a $JabberLib(roster,users) {
	foreach b $JabberLib(roster,users,$a) {
		catch {unset JabberLib(roster,users,$a,$b)}
	}
	catch {unset JabberLib(roster,users,$a)}
  }

  set JabberLib(roster,groups) ""
  set JabberLib(roster,users) ""

  JabberLib:outmsg "<roster><get/></roster>"
}
