#include "common.h"
#include "queue.h"

queue_st *queue_new(unsigned long max)
{
  queue_st *new_queue = 0;

  new_queue = (queue_st *) malloc(sizeof(queue_st));
  if (new_queue == 0)
    return (queue_st *) 0;

  CREATEMUTEX(new_queue->wlock);
  CREATEMUTEX(new_queue->rlock);
  new_queue->count = 0;
  new_queue->max = max;
  new_queue->items = 0;
  return new_queue;
}

short queue_add(queue_st * queue, queue_item * item)
{
  if ((item == 0) || (queue == 0))
    return -1;

  LOCKMUTEX(queue->rlock);	/* Assure atomicity */
  LOCKMUTEX(queue->wlock);

  if (queue->items == 0)
  {
    queue->items = item;
    queue->items->next = queue->items;
    queue->items->prev = queue->items;
  }
  else
  {
    item->next = queue->items;
    item->prev = queue->items->prev;
    queue->items->prev->next = item;
    queue->items->prev = item;
  }

  queue->count++;

  UNLOCKMUTEX(queue->wlock);
  UNLOCKMUTEX(queue->rlock);
  return 1;
}

queue_item *queue_top(queue_st * queue)
{
  if (queue == 0)
    return (queue_item *) 0;

  return queue->items;
}

short queue_del(queue_st * queue, void *data)
{
  queue_item *skip = 0;

  if ((queue == 0) || (data == 0) || (queue->count == 0))
    return -1;

  skip = queue->items;

  LOCKMUTEX(queue->rlock);

  do
  {
    if (skip->data == data)
    {
      /* Found correct item in the queue. */

      LOCKMUTEX(skip->prev->rlock);
      LOCKMUTEX(skip->prev->wlock);
      LOCKMUTEX(skip->next->rlock);
      LOCKMUTEX(skip->next->wlock);

      skip->prev->next = skip->next;
      skip->next->prev = skip->prev;

      UNLOCKMUTEX(skip->next->wlock);
      UNLOCKMUTEX(skip->next->rlock);
      UNLOCKMUTEX(skip->prev->wlock);
      UNLOCKMUTEX(skip->prev->rlock);

      queue_item_del(skip);

      /* Found item out of the loop. */

      break;
    }
    skip = skip->next;
  }
  while (skip != queue->items);

  LOCKMUTEX(queue->wlock);
  queue->count--;
  UNLOCKMUTEX(queue->wlock);

  UNLOCKMUTEX(queue->rlock);

  return 0;
}

short queue_free(queue_st * queue)
{
  if (queue == 0)
    return -1;

  while (queue->count > 0)
  {
    queue_item_free(queue->items);
    queue->count--;
  }

}
