#include "aimt.h"

conn *dispatch(char *p, conn * c, int flag)
{
  conn *c_new = NULL;
  xpt *xml, *next;
  int sock;

  DBG_VAL("dispatch_data", p);

  switch (flag)
  {
  case IO_REMOVE:
    DBG_VAL("removing_connection", c->name);
    if (c == etc.transport)
    {
      ERROR("etherx", "Transport connection failure, exiting");
      /* should start special loop to re-connect to transport with timeout, if reconnect, reconfigure, etc */
      sig_die();
    }
    else
    {
      (int) c_new = -1;
    }
    break;

  case IO_INIT:
    DBG_MSG("Initialization from IO loop");

    /* connect to the local etherx */
    sock = make_netsocket(DEFAULT_ETHERXPORT, jpair_getval(etc.vars, "local"), 1);
    if (sock < 0)
    {
      ERROR("etherx", "Unable to connect to Transport, exiting");
      sig_die();
    }
    c_new = conn_add(NULL, sock);
    c_new->packets = packet_add(c_new->packets, "<?xml version='1.0'?><ether><handshake>", NULL);
    c_new->packets = packet_add(c_new->packets, DEFAULT_ID, NULL);
    c_new->packets = packet_add(c_new->packets, "</handshake>", NULL);
    etc.transport = c_new;	/* store globally */
    break;

  case IO_NORMAL:
    DBG_MSG("Processing Normally");
    if (c->xp == NULL)		/* initialize xp if it hasn't been yet */
      c->xp = xpt_packet_new();
    xpt_packet_data(c->xp, p, strlen(p));

    next = xml = c->xp->packets;
    while (next != NULL)
    {
      xml = next;
      next = next->next;
      xml->next = NULL;		/* single it out */
      if (etc.phase == PHASE_ICQ)
	icq_handle_packet(c, xml);
      else
	handle_packet(c, xml);
    }
    free_xpt(c->xp->packets);
    c->xp->packets = NULL;
    if (c->xp->flag != XPT_POOLFLAG_NONE)
    {
      switch (c->xp->flag)
      {
      case XPT_POOLFLAG_ERROR:
	syslog_debug(SYSLOG_ERROR, "connection", "INVALID_XML_FROM", c->name);
	break;
      case XPT_POOLFLAG_CLOSE:
	DBG_VAL("connection_xml_close", c->name);
	break;
      default:
	DBG_VAL("unknown_xml_error", c->name);
      }
      /* !!! should send error message back about this */
      (int) c_new = -1;
    }
    break;
  }

  return c_new;
}

void handle_packet(conn * c, xpt * x)
{
  xpt *xml, *next;
  char *val = NULL;
  char *type, *to;

  DBG_VAL("handling_normal_packet", c->name);

  type = x->name;
  if (type == NULL)		/* not a valid incoming packet */
  {
    return;
  }

  /* stringpieces->string->xpt to make sure we get xpt in the end */
  next = x->children;
  while (next != NULL)
  {
    if (next->type == XPT_CDATA)
      val = strgrow(val, next->value, 1, 0);
    next = next->next;
  }
  DBG_VAL("transport_packet", val);

  xml = xpt_str(val);
  free(val);

  if (NULL == xml)
  {
    ERROR("template", "Got some bad XML!");
    return;
  }

  if (strcmp(x->name, "r") == 0)
  {
    DBG_MSG("Handling remote packet from transport");
    /* figure out who this message is two, look em up, and forward it */
    to = resolve(xml);
    if(to != NULL)
      forward_packet(to, xml);
    else
      DBG_MSG("couldnt identify sender");
    free_xpt(xml);
    return;
  }

  if (strcmp(x->name, "error") == 0)
  {
    DBG_MSG("Handling bounced packet from transport");
    free_xpt(xml);
    return;
  }

  if (strcmp(x->name, "config") == 0)
  {
    DBG_MSG("Handling config packet from transport");
    config_handle(xml->children);
    /* don't free the config, is referenced throughout apps lifetime */
    return;
  }

  syslog_debug(SYSLOG_ERROR, "etherx", "unknown_packet", x->name);
}

char *resolve(xpt * xml)
{
  char *from;
  char *forkid;

  DBG_MSG("Resolving packet(finding fork id)");

  from = jpair_getval(xml->attributes, "from");
  if (jpair_get(etc.users, from) == NULL)
    return NULL;

  forkid = jpair_getval(etc.forks, from);
  DBG_VAL("from",from);
  DBG_VAL("forkid",forkid);
  if (forkid != NULL)
    return forkid;
  else
    return new_fork(from);
}

char *new_fork(char *user)
{
  static int counter = 1;
  char newid[20], *str;
  jpair *var;
  /* fork, set new name in etc.vars, set globals, reconnect to transport and set etc.transport->id, set etc.phase */

  DBG_MSG("New fork");

  sprintf(newid, "icq%d", counter++);

  var = jpair_get(etc.forks, user);
  if(var != NULL)
  {
    free(var->val);
    var->val = strdup(newid);
  }else{
    etc.forks = jpair_new(etc.forks, user, newid, 0);
  }

  if (fork() > 0)
  {
    sleep(1);
    return strdup(newid);
  }
  /* we are now the new forkd process, setup libicq stuff */

  DBG_VAL("forked",newid);

  var = jpair_get(etc.vars, "name");
  free(var->val);
  var->val = strdup(newid);

  var = jpair_get(etc.users, user);
  ICQ_Register_Callback(EVENT_LOGIN, EventLogin);
  ICQ_Register_Callback(EVENT_DISCONNECT, EventDisconnect);
  ICQ_Register_Callback(EVENT_MESSAGE, EventMessage);
  ICQ_Register_Callback(EVENT_ONLINE, EventChangeStatus);
  ICQ_Register_Callback(EVENT_OFFLINE, EventChangeStatus);
  ICQ_Register_Callback(EVENT_STATUS_UPDATE, EventChangeStatus);
  ICQ_Register_Callback(EVENT_SEARCH_RESULTS, EventSearchResults);
  ICQ_Register_Callback(EVENT_INFO, EventInfo);
  ICQ_Register_Callback(EVENT_EXT_INFO, EventExtInfo);
  ICQ_Debug(ICQ_VERB_INFO);

  UIN = var->flag;
  strcpy(passwd, var->val);
  set_status = STATUS_ONLINE;
  strcpy(server, "icq.mirabilis.com");
  remote_port = 4000;
  Num_Contacts = 0;

  dispatch(NULL, NULL, IO_INIT);	/* lazy way to reconnect to etherx */
  str = make_tag_quick("alias", newid);
  etc.transport->packets = packet_add(etc.transport->packets, str, NULL);
  free(str);
  fcntl(etc.transport->id, F_SETFL, O_NONBLOCK);
  etc.phase = PHASE_ICQ;
  etc.user = strdup(user);
  etc.status = JSTATUS_ONLINE;

  icq_main();			/* never returns */
}

void forward_packet(char *to, xpt * x)
{
  char *ret = NULL;

  DBG_VAL("forwarding packet to",to);

  ret = strgrow(ret, "<r from='", 1, 0);
  ret = strgrow(ret, jpair_getval(etc.vars, "name"), 1, 0);
  ret = strgrow(ret, "' to='", 1, 0);
  ret = strgrow(ret, to, 1, 0);
  ret = strgrow(ret, "'><![CDATA[", 1, 0);
  ret = strgrow(ret, xpt2str(x), 1, 1);
  ret = strgrow(ret, "]]></r>", 1, 0);

  etc.transport->packets = packet_add(etc.transport->packets, ret, "");

  free(ret);
}
