// laim_aim.cpp
// AIM specific routines
// LAIM
// Copyright (C) 1998 Jae Jang <akubi@stones.com>
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
// ------------------------------------------------------------            
#include <iostream.h>
#include <fstream.h>
// socket related includes
#include <errno.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <unistd.h>
#include <fcntl.h>
#include "laim_core.h"
string word_to_str(unsigned short);

AIM_session::AIM_session()
{
	proto_id = AIM;
	server_ip = "login.oscar.aol.com";
	server_port = 5190;
	connected = false;
	mesg_biff = false;
	screen_name = "";
	password = "";
	error_num = 0x00;
	error_str = "";
	notice_num = IDLE_MESG;
	notice_str = "";
	send_buffer = recv_buffer = "";
	send_seq = recv_seq = 1; 
	cookie = "";
}

AIM_session::AIM_session(string s, string p)
{
	proto_id = AIM;
	server_ip = "login.oscar.aol.com";
	server_port = 5190;
	connected = false;
	mesg_biff = false;
	screen_name = s;
	password = p;
	error_num = 0x00;
	error_str = "";
	notice_num = IDLE_MESG;
	notice_str = "";
	send_buffer = recv_buffer = "";
	send_seq = recv_seq = 1; 
	cookie = "";
}

AIM_session::~AIM_session()
{
	notice_num = NETSTAT_MESG;
	notice_str = "Closing socket.";
	close(socket_num);
}

// this will increment IM_session::recv_seq,
// finds channel from FLAP header and add 'char channel' to
// the string of raw data and return the whole string (length==len.data+1)
void AIM_session::recv_buf()
{
	int recved;
	char header[3]="";
	char channel; // FLAP channel

	// get 2 byte FLAP_id, presumably 0x2aXX
	recved = recv(socket_num, header, 2, 0);
	//while(recved = recv(socket_num, header, 2, 0)==-1)) {
	//}
	if (recved == -1) {
		recv_buffer = "";
		return;
	}
	channel = header[1];
#ifdef LAIM_DEBUG
	cerr << "(>>) " << header << endl;
	cerr << "(CH) channel resolved as " << int(channel) << endl;
#endif
	// get sequence number from server
	unsigned short seq;
	recved = recv(socket_num, &seq, 2, 0);
	// the following won't (probably) hold for non intel platforms
	// it flips byte order... from intel (presumed to be the platform)
	// Least SB to Most SB
	seq=htons(seq);
	if (!connected) {
		recv_seq = seq;
		connected = true;
	}
	else if (seq != (++recv_seq)) {
		notice_num = ERROR_MESG;
		notice_str = "Server packet out of sync... oh well. ^_^'";
		recv_seq = seq;
	}
#ifdef LAIM_DEBUG
	cerr << "(SQ) " << hex << seq << ':' << recv_seq << endl;
#endif
	// get packet length (excluding this 6 byte header)
	unsigned short p_len;
	recved = recv(socket_num, &p_len, 2, 0);
	// the following won't (probably) hold for non intel platforms
	// it flips byte order... from intel (presumed to be the platform)
	// Least SB to Most SB
	p_len=htons(p_len);
#ifdef LAIM_DEBUG
	cerr << "(PS) data size: " << p_len << " bytes" << endl;
#endif
	char* tmp_buf = new char[p_len];
	recved = recv(socket_num, tmp_buf, p_len, 0);
	recv_buffer = string(tmp_buf, p_len);
	delete tmp_buf;
#ifdef LAIM_DEBUG
	cerr << "(**) " << recved << " bytes received." << endl;
#endif
	recv_buffer += channel;
#ifdef LAIM_DEBUG
	cerr << "(DT) " << recv_buffer << endl;
#endif
}

// flushes whatever's in the send_buffer while inserting the
// correct FLAP header automatically. chan is FLAP channel.
void AIM_session::send_buf(char chan)
{
	string send_s;
	// header
	send_s = '\x2a';
	send_s += chan;
	send_s += word_to_str(send_seq++);
	send_s += word_to_str(send_buffer.length());
	// data
	send_s += send_buffer;
#ifdef LAIM_DEBUG
	cerr << "(<<) " << send_s << endl;
	cerr << "(CH) channel resolved as " << int(chan) << endl;
	cerr << "(SQ) " << hex << int(send_seq-1) << endl;
#endif
	int sent;
	sent = send(socket_num, send_s.data(), send_s.length(), 0);
#ifdef LAIM_DEBUG
	cerr << "(**) " << sent << " bytes sent." << endl;
#endif
}

// returns -1 on error
// server_ip is changed to BOS IP address on successful OSCAR xmission
void AIM_session::login()
{
	char channel;
	int s_len;
	string s;

	// connection acknowledge
	recv_buf();
	s_len = recv_buffer.length();
	channel = recv_buffer[s_len-1];
	if (channel != 0x01) {
#ifdef LAIM_DEBUG
		cerr << "channel is " << channel << " while 0x01 was expected.\n";
#endif
		return;
	}
	if (recv_buffer.find(word_to_str(0x0000)+word_to_str(0x0001)) != 0) {
#ifdef LAIM_DEBUG
		cerr << "0x0000 0001 expected\n";
#endif
		return;
	}
	// send Auth request packet.
	// write all data and add the header after.
	send_buffer = word_to_str(0x0000) + word_to_str(0x0001);
	send_buffer += TLV(0x01, screen_name.length(), screen_name);
	send_buffer += TLV(0x02, password.length(), passwd_crypt(password));
/*	s = "AOL Instant Messenger (TM) version 1.1.19 for Java"
		" built 03/24/98, freeMem 449616 totalMem 1048568, x86,"
		" Linux, 2.0.30, Sun Microsystems Inc., ported by"
		" Randy Chapman and Steve Cyrne, root:08/30/97-09:42, 45.3,"
		" Sun Aug 09 21:57:00 EDT 1998"; */
	// AOL Instant Messenger is a trademark of AOL not me
	s = "LAIM Copyright (c)1998 <akubi@stones.com>"
		" version 1.1.19 built 08/10/98, x86, Linux;"
		" LAIM's An Instant Messenjer";
	send_buffer += TLV(0x03, s.length(), s);
	// "unknown section" T=0x16
	send_buffer += TLV(0x16, 0x02, word_to_str(0x0001));
	send_buffer += TLV(0x17, 0x02, word_to_str(LAIM_MAJOR_VERSION));
	send_buffer += TLV(0x18, 0x02, word_to_str(LAIM_MINOR_VERSION));
	send_buffer += TLV(0x1a, 0x02, word_to_str(LAIM_BUILD));
	send_buffer += TLV(0x0e, 0x02, "us");
	send_buffer += TLV(0x0f, 0x02, "en");
	send_buffer += TLV(0x09, 0x02, word_to_str(0x0015));
	send_buf(0x01);
	// recv Auth response packet.
	recv_buf();
	s_len = recv_buffer.length();
	channel = recv_buffer[s_len-1];
	if (channel != 0x04) {
#ifdef LAIM_DEBUG
		cerr << "channel is " << channel << " while 0x04 was expected.\n";
#endif
		return;
	}
	//interpret it!
	OSCAR_TLV_decode();
	// logout from oscar now that we have the redirect IP and cookie
	send_buffer = word_to_str(0x0000);
	send_buf(0x04);
	close(socket_num);
	connected = false;
	// check if any errors occurred
	if (error_num != 0x00) {
		socket_num = -1;
	} 
	else {
	// connect to BOS
		BOS_login();
	}
}

// called by IM_session::login() after oscar login/logout
// this monstrosity handles BOS login, including a call to get
// the buddy list. After this the frontend is free to handle
// "live events", namely send/recv IMs.
void AIM_session::BOS_login()
{
	struct sockaddr_in my_addr;
	// do "standard" bsd-socket opening crap
	// mostly from beej's socket programming tutorial
	socket_num = socket(AF_INET, SOCK_STREAM, 0);
	my_addr.sin_family = AF_INET;
	my_addr.sin_port = htons(server_port);
	my_addr.sin_addr.s_addr = inet_addr(server_ip.c_str());
	bzero(&(my_addr.sin_zero), 8);
#ifdef LAIM_DEBUG
	cerr << "contacting host " << server_ip << ':' << server_port << endl;
#endif
	if (connect(socket_num,(struct sockaddr *)&my_addr, sizeof(struct
		sockaddr)) == -1) {
		perror("connect");
		exit(1);
	}
	char channel;
	int s_len;
	string s;

	recv_buf();
	s_len = recv_buffer.length();
	channel = recv_buffer[s_len-1];
	if (channel != 0x01) {
#ifdef LAIM_DEBUG
		cerr << "channel is " << channel << " while 0x01 was expected.\n";
#endif
		return;
	}
	if (recv_buffer.find(word_to_str(0x0000)+word_to_str(0x0001)) != 0) {
#ifdef LAIM_DEBUG
		cerr << "0x0000 0001 expected\n";
#endif
		return;
	}
	// send BOS signon request
	send_buffer = word_to_str(0x0000) + word_to_str(0x0001);
	send_buffer += TLV(0x06, cookie.length(), cookie);
	send_buf(0x01);
	// recv BOS Host-ready
	recv_buf();
	s_len = recv_buffer.length();
	channel = recv_buffer[s_len-1];
	if (channel != 0x02) {
		cerr << "channel is " << channel << " while 0x02 was expected.\n";
		return;
	}
	SNAC_decode();
	// send rate request
	SNAC_packet S_pkt(0x01,0x06,0x00,0x00,0x1015);	//1015 is completely rand
	send_buffer = S_pkt.packet();
	send_buf(0x02);
	// recv rate responce
	recv_buf();
	s_len = recv_buffer.length();
	channel = recv_buffer[s_len-1];
	if (channel != 0x02) {
		cerr << "channel is " << channel << " while 0x02 was expected.\n";
		return;
	}
	SNAC_decode();
	// send (????) 0x01:0x08 rate ack
	S_pkt.update(0x01,0x08,0x00,0x00,0x1016);
	S_pkt.data = word_to_str(0x0001) + word_to_str(0x0002)
					+ word_to_str(0x0003) + word_to_str(0x0004);
	send_buffer = S_pkt.packet();
	send_buf(0x02);
	// send (????) 0x01:0x14 privacy flags 
	S_pkt.update(0x01,0x14,0x00,0x00,0x1017);
	S_pkt.data = word_to_str(0x0000) + word_to_str(0x0003);
	send_buffer = S_pkt.packet();
	send_buf(0x02);
	// send nickname info request
	S_pkt.update(0x01,0x0e,0x00,0x00,0x1018);
	send_buffer = S_pkt.packet();
	send_buf(0x02);
	// send Ad server service request. Take it out later...
	//S_pkt.update(0x01,0x04,0x00,0x00,0x1019);
	//S_pkt.data = word_to_str(0x0005); // 0x0005 is ad server genus
	//send_buffer = S_pkt.packet();
	//send_buf(0x02);
	// send BOS rights req
	S_pkt.update(0x09,0x02,0x00,0x00,0x101a);
	send_buffer = S_pkt.packet();
	send_buf(0x02);
	// send Buddylist rights req
	S_pkt.update(0x03,0x02,0x00,0x00,0x101b);
	send_buffer = S_pkt.packet();
	send_buf(0x02);
	// send 0x02 rights req
	S_pkt.update(0x02,0x02,0x00,0x00,0x101c);
	send_buffer = S_pkt.packet();
	send_buf(0x02);
	// send IM parameter info req
	S_pkt.update(0x04,0x04,0x00,0x00,0x101d);
	send_buffer = S_pkt.packet();
	send_buf(0x02);
	// whew... now recv all replies. ;_;
	// recv rate responce
	for (int i = 1; i <= 5; i++) {
		recv_buf();
		s_len = recv_buffer.length();
		channel = recv_buffer[s_len-1];
		if (channel != 0x02) {
#ifdef LAIM_DEBUG
			cerr << "channel is " << channel << " while 0x02 was expected.\n";
#endif
			return;
		}
		SNAC_decode();
	}
	// send... more....
	// send optional (????) stuff
	S_pkt.update(0x09,0x04,0x00,0x00,0x101e);
	S_pkt.data = word_to_str(0x0000) + word_to_str(0x001f);
	send_buffer = S_pkt.packet();
	send_buf(0x02);
	S_pkt.update(0x09,0x07,0x00,0x00,0x101f);
	send_buffer = S_pkt.packet();
	send_buf(0x02);
	// send buddy list
	S_pkt.update(0x03,0x04,0x00,0x00,0x1020);
	S_pkt.data = contacts_list();
	send_buffer = S_pkt.packet();
	send_buf(0x02);
	// send user profile
	S_pkt.update(0x02,0x04,0x00,0x00,0x1021);
	s = profile();
	S_pkt.data = TLV(0x01, 0x0d, "text/x-aolrtf") + TLV(0x02, s.length(), s);
	send_buffer = S_pkt.packet();
	send_buf(0x02);
	// send set initial ICBM parameter
	S_pkt.update(0x04,0x02,0x00,0x00,0x1022);
	S_pkt.data  = word_to_str(0x01) + word_to_str(0x00) + word_to_str(0x03);
	S_pkt.data += word_to_str(0x1f3f) + word_to_str(0x03e7);
	S_pkt.data += word_to_str(0x03e7) + word_to_str(0x00) + word_to_str(0x64);
	send_buffer = S_pkt.packet();
	send_buf(0x02);
	// send client online/ready
	S_pkt.update(0x01,0x02,0x00,0x00,0x1023);
	S_pkt.data  = word_to_str(0x01) + word_to_str(0x02) + word_to_str(0x01)
		+ word_to_str(0x13) + word_to_str(0x09) + word_to_str(0x01)
		+ word_to_str(0x01) + word_to_str(0x01) + word_to_str(0x03)
		+ word_to_str(0x01) + word_to_str(0x01) + word_to_str(0x01)
		+ word_to_str(0x04) + word_to_str(0x01) + word_to_str(0x01)
		+ word_to_str(0x01) + word_to_str(0x02) + word_to_str(0x01)
		+ word_to_str(0x01) + word_to_str(0x01) + word_to_str(0x08)
		+ word_to_str(0x01) + word_to_str(0x01) + word_to_str(0x01)
		+ word_to_str(0x06) + word_to_str(0x01) + word_to_str(0x01)
		+ word_to_str(0x01) + word_to_str(0x0a) + word_to_str(0x01)
		+ word_to_str(0x01) + word_to_str(0x01) + word_to_str(0x0b)
		+ word_to_str(0x01) + word_to_str(0x01) + word_to_str(0x01);
	send_buffer = S_pkt.packet();
	send_buf(0x02);
}

// returns length byte, SN, length byte, SN, ... for all SNs in buddy list
// in one string. also updates IM_session::contacts
string AIM_session::contacts_list()
{
	// I will attempt to parse AIM's ~/.aim/AIM.cfg file...
	string AIM_cfg_fname = home_dir + "/.aim/AIM.cfg";
	ifstream AIM_cfg(AIM_cfg_fname.c_str());
	if (!AIM_cfg) {
		cout << "Unable to read ~/.aim/AIM.cfg. Read the README file" << endl;
		exit(2);
	}
	string r = "";
	string s;
#ifdef LAIM_DEBUG
	cerr << "reading " << AIM_cfg_fname << endl;
#endif
	do {
		AIM_cfg >> s;
	} while (s != "Buddy");
#ifdef LAIM_DEBUG
	cerr << "inside Buddy section\n";
#endif
	AIM_cfg >> s;
	if (s != "{") {
#ifdef LAIM_DEBUG
		cerr << "Buddy section: { expected\n";
#endif
		return "";
	}
	do {
		AIM_cfg >> s;
	} while (s != "List"); // get to the Buddy:list section
#ifdef LAIM_DEBUG
	cerr << "inside List section\n";
#endif
	AIM_cfg >> s;
	if (s != "{") {
#ifdef LAIM_DEBUG
		cerr << "List section: { expected\n";
#endif
		return "";
	}
	AIM_cfg >> s;
	while (s != "}") {
#ifdef LAIM_DEBUG
		cerr << "inside " << s << " section\n";
#endif
		AIM_cfg >> s;
		if (s != "{") {
#ifdef LAIM_DEBUG
			cerr << s << " section: { expected\n";
#endif
			return "";
		}
		// yay, we're in... now for the list of screen names.
		// one thing to watch out for is that if the screen name contains
		// spaces, the entire name is contained in quotes.
		AIM_cfg >> s;
		while (s != "}") {
			if (s[0] == '\"') {
				// get entire name into s
				while (s[s.length()-1] != '\"') {
					string tmp;
					AIM_cfg >> tmp;
					s += (' ' + tmp);	// this is bad... I'm assuming
										// there's only one space.
				}
				// remove the quotes
				string tmp(s,1,s.length()-2);
				s = tmp;
			}
			char len = s.length();	// buffer overflow problem coming(????)
#ifdef LAIM_DEBUG
			cerr << "adding " << s << " to buddy list" << endl;
#endif
			contact.add(AIM, s);
#ifdef LAIM_DEBUG
			cerr << "Buddy list has " << contact.person.size() << " people\n";
#endif
			r += (len + s);
			AIM_cfg >> s;
		}
#ifdef LAIM_DEBUG
		cerr << "adding a ghost" << endl;
#endif
//		s = "";
//		contact.add(GHOST,s);
		contact.add(GHOST, "", "", hidden, 1015);
		AIM_cfg >> s;
	}
	AIM_cfg.close();
	return r;
}

// gets profile from a text file and returns it. duh
string AIM_session::profile()
{
	// read AIM's ~/.aim/Info.sig file...
	string AIM_sig_fname = home_dir + "/.aim/Info.sig";
	ifstream AIM_sig(AIM_sig_fname.c_str());
	if (!AIM_sig) {
		cout << "Unable to read ~/.aim/Info.sig. Read the README file" << endl;
		exit(3);
	}
	string r = "";
#ifdef LAIM_DEBUG
	cerr << "reading " << AIM_sig_fname << endl;
#endif
	char c;
	while (AIM_sig.get(c))
		r += c;
	AIM_sig.close();
	return r;
}

// becareful when calling this with data as char*.
// char* data must NOT contain '\0' or the "string"
// will end there. for passing data of the form 0x0001,
// use the function word_to_str(unsigned short).
string AIM_session::TLV(unsigned short id, unsigned short len, string data)
{
	string r;
	r = word_to_str(id);
	r += word_to_str(len);
	r += data;
	return r;
}
	
string AIM_session::passwd_crypt(string plaintext)
{
	string xor_mask =
	"\xf3\xb3\x6c\x99\x95\x3f\xac\xb6\xc5\xfa\x6b\x63\x69\x6c\xc3\x9f";
	string r = ""; // de/encoded password to be returned
	for (unsigned int i=1; i <= plaintext.length(); i++)
		r += (plaintext[i-1] ^ xor_mask[i-1]);
	return r;
}

// takes first 2 bytes out of recv_buffer and returns it's u_short value
// deletes the 2 bytes and thus shortens recv_buffer by 2 bytes.
unsigned short IM_session::get_word()
{
	char foo = recv_buffer[0];
	char bar = recv_buffer[1];
	recv_buffer = string(recv_buffer,2,string::npos);
	return ((foo << 8) + bar);
}

void AIM_session::OSCAR_TLV_decode()
{
	//parse by TLV
	// intel platform byte ordering scheme
	while (recv_buffer.length() > 1) { // >1 not >0 because of the trailing
									   // channel byte
		unsigned short T, L;  // Type, Length
		T = get_word(); // recall, get_word deletes the first two bytes
		L = get_word(); // after use
		string V(recv_buffer, 0, L);
		recv_buffer = string(recv_buffer, L, string::npos);
		switch (T) {
			case 0x01 : // "beautified" screen name
				screen_name = V;
#ifdef LAIM_DEBUG
				cerr << "screen name is " << screen_name << endl;
#endif
				break;
			case 0x04 : // error message URL
				error_str = V;
#ifdef LAIM_DEBUG
				cerr << "error, server says to visit: " << error_str << endl;
#endif
				break;
			case 0x05 : // BOS IP
				server_ip = V;
#ifdef LAIM_DEBUG
				cerr << "redirect is to " << server_ip << endl;
#endif
				break;
			case 0x06 : // Authorization Cookie
				cookie = V;
#ifdef LAIM_DEBUG
				cerr << "received cookie" << endl;
#endif
				break;
			case 0x08 : // error number
				error_num = (V[0] << 8) + V[1];
#ifdef LAIM_DEBUG
				switch (error_num) {
					case 0x01 : // Invalid screen name
						cerr << "invalid screen name" << endl;
						break;
					case 0x05 : // Invalid password
						cerr << "invalid password" << endl;
						break;
					case 0x18 : // too many logins?
						cerr << "logged on too many times on a " <<
								"set time interval" << endl << 
								"try again later" << endl;
						break;
					default : // unknown
						cerr << "unknown error. please email me the"
							" the error number (" << hex << error_num <<
							")" << endl;
				}
#endif
				break;
			case 0x11 : // Email Address
				email = V;
#ifdef LAIM_DEBUG
				cerr << "email address is " << email << endl;
#endif
				break;
			case 0x13 : // registration status ?? (word) intel
				status = (V[0] << 8) + V[1];
#ifdef LAIM_DEBUG
				cerr << "registration status is " << status << endl;
				break;
			default :
				cerr << "Unknown TLV\n";
#endif
		}
		
	}
}

