// LAIM
// Copyright (C) 1998 Jae Jang <akubi@stones.com>
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
// ------------------------------------------------------------            
#include <iostream.h>
// socket related includes
#include <errno.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <unistd.h>
#include "laim_core.h"
// (????)
//#include <time.h>

string word_to_str(unsigned short);
void Buddy_TLV_decode(int count, string& st);

void AIM_session::SNAC_decode()
{		
	SNAC_packet S_pkt(recv_buffer);
	recv_buffer = string(recv_buffer, 10, string::npos);
	// just temporary variables for use in the big switch statement
	string s;
	string tmp;
	unsigned char c;
	unsigned short i;
	bool autoresp = false;
	Contact* bud;
	notice_str = "";
	if (S_pkt.specie == 0x01) {
		notice_num = ERROR_MESG;
		notice_str = "error on genus #" + word_to_str(S_pkt.genus);
		return;
	}
	switch (S_pkt.genus) {
		case 0x01 : // general service controls
			notice_num = SERVER_MESG;
			notice_str = "General service controls\n";
			switch (S_pkt.specie) {
				case 0x03 : // server ready signal
					s  = word_to_str(0x0001) + word_to_str(0x0002);
					s += word_to_str(0x0003) + word_to_str(0x0004);
					s += word_to_str(0x0006) + word_to_str(0x0008);
					s += word_to_str(0x0009) + word_to_str(0x000a);
					s += word_to_str(0x000b) + word_to_str(0x000c);
					if (recv_buffer.find(s) == 0)
						notice_str += "Server waiting...";
					break;
				case 0x07 : // rate information ??
					notice_str += "Rate information received.";
					break;
				case 0x0a : // rate violation
					notice_str += "Calm down!! The server doesn't ";
					notice_str += "like people hammering IMs.";
					break;
				case 0x0f : // info on our screen name
					notice_str += "Screen name information received.";
					break;
				case 0x10 : // warning
					notice_str += "Someone just warned you.";
					break;
				case 0x12 : // migration notice
					notice_str += "Migration notice...";
					break;
				case 0x13 : // ???? server sends this after BOS login...
					notice_str += "...";
					break;
#ifdef LAIM_DEBUG
				default :
					cerr << "Unknown specie #" << S_pkt.specie
						<< " for genus #" << S_pkt.genus << endl;
#endif
					/*
					notice_str += ("Unknown specie #" 
						+ word_to_str(S_pkt.specie));
					notice_str += (" for genus #" + word_to_str(S_pkt.genus));
					*/
			}
			break;
		case 0x02 : // Locate service
			notice_num = SERVER_MESG;
			notice_str = "Locate service\n";
			switch (S_pkt.specie) {
				case 0x03 : // rights resp
					notice_str += "Locate service rights response";
					break;
				case 0x06 : // user info
					c = recv_buffer[0];
					s = string(recv_buffer, 1, c); // s contains SN
					s = string(recv_buffer, 1+c, string::npos); // rm SN
					i = ::get_word(s); // 0x0000
					i = ::get_word(s); // num of following TLVs
					Buddy_TLV_decode(i, s);
					// the buddy portion of s has been deleted by decode
					i = ::get_word(s); // 0x0001
					i = ::get_word(s); // 0x0D
					s = string(s, i, string::npos); // skip text/x-aolrtf (0x0D)
					i = ::get_word(s); // 0x0002
					i = ::get_word(s); // length of profile
					s = string(s, 0, i); // s is the profile
					notice_str = s;
					break;
#ifdef LAIM_DEBUG
				default :
					cerr << "Unknown specie #" << S_pkt.specie
						<< " for genus #" << S_pkt.genus << endl;
#endif
					/*
					notice_str += ("Unknown specie #" 
						+ word_to_str(S_pkt.specie));
					notice_str += (" for genus #" + word_to_str(S_pkt.genus));
					*/
			}
			break;
		case 0x03 : // Buddylist
			notice_num = BUDDY_LIST_MESG;
			notice_str = "";
			switch (S_pkt.specie) {
				case 0x03 : // buddy list rights response
					notice_str += "Buddy list rights response";
					break;
				case 0x0b : // buddy signed on (or is currently on)
					c = recv_buffer[0];	// length of SN
					s = string(recv_buffer, 1, c); // tmp contains SN
					bud = contact.find_contact(s);	// C_LI bud
						// to the Contact class of s==SN
					contact.update(bud);
#ifdef LAIM_DEBUG
					cerr << bud->alias << " signed on\n";
#endif
					if ( (bud->status == signon) ||
						 (bud->status == online) ) {
						 // server is just sending refresh
					} 
					else {
						bud->status = signon;
						notice_str += s + " has signed on.";
					}
					break;
				case 0x0c : // buddy signed off
					c = recv_buffer[0]; // length of SN
					s = string(recv_buffer, 1, c); // tmp contains SN
					bud = contact.find_contact(s);	// C_LI bud 
						// to the Contact class of s==SN
					contact.update(bud);
#ifdef LAIM_DEBUG
					cerr << bud->alias << " signed off.\n";
#endif
					if ( (bud->status == signoff) ||
						 (bud->status == offline) ) {
						 // server is just sending refresh
					} 
					else {
						bud->status = signoff;
						notice_str += s + " has signed off.";
					}
					break;
#ifdef LAIM_DEBUG
				default :
					cerr << "Unknown specie #" << S_pkt.specie
						<< " for genus #" << S_pkt.genus << endl;
#endif
					/*
					notice_str += ("Unknown specie #" 
						+ word_to_str(S_pkt.specie));
					notice_str += (" for genus #" + word_to_str(S_pkt.genus));
					*/
			}
			break;
		case 0x04 : // IM service
			notice_num = SERVER_MESG;
			notice_str = "";
			switch (S_pkt.specie) {
				case 0x03 : // rights resp
					notice_str += "IM service rights response";
					break;
				case 0x07 : // recv IM
					// skip first 8 bytes (unknown purpose)
					// 0x0001
					// SN length
					c = recv_buffer[10];
					// get SN of sender
					s = string(recv_buffer, 11, c);
#ifdef LAIM_DEBUG
					cerr << s << " sent you a message\n" << endl;
#endif
					bud = contact.find_contact(s);	// C_LI bud 
						// to the Contact class of s==SN
					if ((bud->proto_id) == GHOST) {
						// someone not on buddy list has sent an IM
						// create a new Contact object
						contact.del(""); // remove ghost from end
						// should be able to insert next to last
						// but I am too lazy to look up how...
						// it's 1:20 AM
						contact.add(AIM,s,s,online,0);
						notice_num = BUDDY_LIST_MESG;
						contact.add(GHOST, "", "", hidden, 1015);
						bud = contact.find_contact(s);
					}
					notice_str = '*' + s + "* ";
					contact.update(bud);
					s = string(recv_buffer, 11 + c, 4);
					i = ::get_word(s);
					i = ::get_word(s);
					s = string(recv_buffer, 11 + c + 4, string::npos);
					Buddy_TLV_decode(i, s);
					// the buddy portion of s has been deleted by decode
					i = ::get_word(s); // 0x0002 || 0x0004
					if (i == 0x0004) {
						i = ::get_word(s); // 0x0000
						autoresp = true;
						i = ::get_word(s); //0x0002
					}
#ifdef LAIM_DEBUG
					if (i != 0x0002) {
						cerr << "0x0002 error\n";
					}
#endif
					i = ::get_word(s); // mesg length + 0x0D
					i = ::get_word(s); // 0x0501
#ifdef LAIM_DEBUG
					if (i != 0x0501)
						cerr << "IM: 0x0501 != 0x" << hex << i << endl;
#endif
					i = ::get_word(s); // L as in TLV
					s = string(s, i + 8, s.length()-(i + 8 + 1)); 
						// get rid of 13 bytes and the trailing 0x02
					// s is now the text/x-aolrtf mesg
					if (!autoresp)
						bud->mesg = s;
					else
						bud->mesg = "Auto responce: " + s;
					mesg_biff = true;
					break;
#ifdef LAIM_DEBUG
				default :
					cerr << "Unknown specie #" << S_pkt.specie
						<< " for genus #" << S_pkt.genus << endl;
#endif
					/*
					notice_str += ("Unknown specie #" 
						+ word_to_str(S_pkt.specie));
					notice_str += (" for genus #" + word_to_str(S_pkt.genus));
					*/
			}
			break;
		case 0x09 :
			notice_num = SERVER_MESG;
			notice_str = "";
			switch (S_pkt.specie) {
				case 0x03 : // BOS rights response
					notice_str += "Client rights response";
					break;
#ifdef LAIM_DEBUG
				default :
					cerr << "Unknown specie #" << S_pkt.specie
						<< " for genus #" << S_pkt.genus << endl;
#endif
					/*
					notice_str += ("Unknown specie #" 
						+ word_to_str(S_pkt.specie));
					notice_str += (" for genus #" + word_to_str(S_pkt.genus));
					*/
			}
			break;
		case 0x0a :	// search by email
			notice_num = SEARCH_RESULT_MESG;
			notice_str = "";
			switch (S_pkt.specie) {
				case 0x01 : // error
					notice_num = ERROR_MESG;
					notice_str = "email address not found";
					break;
				case 0x03 : // success. result = SNlen + SN + SNlen + SN ...
					s="";
					while (recv_buffer.length()>4) {
						i = ::get_word(recv_buffer); // 0x0001
						s += string(recv_buffer,0,2);
						i = ::get_word(recv_buffer); // SNlen
						s += string(recv_buffer,0,i);
						recv_buffer = string(recv_buffer,i,string::npos);
					}
					notice_str = s;
					break;
#ifdef LAIM_DEBUG
				default :
					cerr << "Unknown specie #" << S_pkt.specie
						<< " for genus #" << S_pkt.genus << endl;
#endif
			}
			break;
		case 0x0b :
			notice_num = SERVER_MESG;
			notice_str = "^_^;;";
			break;
		default :
			notice_num = ERROR_MESG;
#ifdef LAIM_DEBUG
			cerr << "Unknown genus #" << S_pkt.genus << endl;
#endif
/*			notice_str = ("Unknown genus #" + word_to_str(S_pkt.genus));
*/
	}
}
					
SNAC_packet::SNAC_packet(unsigned short g, unsigned short s, char f1, char f2,
							long req)
{
	genus = g;
	specie = s;
	flag[0] = f1;
	flag[1] = f2;
	req_id = req;
	data="";
}

SNAC_packet::SNAC_packet(string s)
{
	// intel
	genus = (s[0] << 8) + s[1];
	specie = (s[2] << 8) + s[3];
	flag[0] = s[4];
	flag[1] = s[5];
	req_id = (s[6] << 24) + (s[7] << 16) + (s[8] << 8) + s[9];
#ifdef LAIM_DEBUG
	cerr << "SNAC header: " << hex << genus << ':' << specie << ':' << 
		flag[0] << ':' << flag[1] << ':' << req_id << endl;
#endif LAIM_DEBUG
	string tmp(s, 10, s.length()-11);
	data = tmp;
#ifdef LAIM_DEBUG
	cerr << "SNAC data: " << data << endl;
#endif LAIM_DEBUG
}

void SNAC_packet::update(unsigned short g, unsigned short s, char f1, char f2,
							long req)
{
	genus = g;
	specie = s;
	flag[0] = f1;
	flag[1] = f2;
	req_id = req;
	data="";
}

void SNAC_packet::decode(string s)
{
	// intel
	genus = (s[0] << 8) + s[1];
	specie = (s[2] << 8) + s[3];
	flag[0] = s[4];
	flag[1] = s[5];
	req_id = (s[6] << 24) + (s[7] << 16) + (s[8] << 8) + s[9];
#ifdef LAIM_DEBUG
	cerr << "SNAC header: " << hex << genus << ':' << specie << ':' << 
		flag[0] << ':' << flag[1] << ':' << req_id << endl;
#endif LAIM_DEBUG
	string tmp(s, 10, s.length()-11);
	data = tmp;
#ifdef LAIM_DEBUG
	cerr << "SNAC data: " << data << endl;
#endif LAIM_DEBUG
}

string SNAC_packet::packet()
{
	string r;
	r = word_to_str(genus) + word_to_str(specie);
	r += flag[0];
	r += flag[1];
	r += word_to_str((req_id >> 16)) + word_to_str(req_id & 0xFFFF);
	r += data;
	return r;
}

// interprets count number of TLVs contained in st
void Buddy_TLV_decode(int count, string& st)
{
	//parse by TLV
	// intel platform byte ordering scheme
	for (int ind = 1; ind <= count; ind++) {
		unsigned short T, L;  // Type, Length
		T = get_word(st); // recall, get_word deletes the first two bytes
		L = get_word(st); // after use
		string V(st, 0, L);
		st = string(st, L, string::npos);
		unsigned short i;
		time_t t; // num seconds since 00:00:00 Jan 1 1970
					// LAIM is now officially Y2034(?) non-compliant ^_^
		//struct tm* t_s; // to get localtime(time_t t)
		string s;
		switch (T) {
			case 0x01 : // "class" 0x10 free 0x04 AOL
				i = get_word(V);
#ifdef LAIM_DEBUG
				cerr << "sender is using ";
				switch (i) {
					case 0x10 :
						cerr << "non-AOL." << endl;
						break;
					case 0x04 :
						cerr << "AOL." << endl;
						break;
					default :
						cerr << "an unknown version of AIM." << endl;
				}
#endif
				break;
			case 0x02 : // member since
				i = get_word(V);
				t = (i << 16) + get_word(V);
				//t_s = localtime(&t);
				//s = asctime(t_s);
				//cerr << "member since " << s;
				break;
			case 0x03 : // online since
				i = get_word(V);
				t = (i << 16) + get_word(V);
				//t_s = localtime(&t);
				//s = asctime(t_s);
				//cerr << "online since " << s;
				break;
			case 0x04 : // idle time
				i = get_word(V);
				break;
#ifdef LAIM_DEBUG
			default :
				cerr << "Unknown TLV\n";
#endif
		}
		
	}
}

unsigned short get_word(string& s)
{
	if (s.length() > 1) {
		char foo = s[0];
		char bar = s[1];
		s = string(s,2,string::npos);	// delete the first two bytes
		return ((foo << 8) + bar);
	}
	else return 0;
}

