// laim_core.cpp
// interface to LAIM backend.
// LAIM
// Copyright (C) 1998 Jae Jang <akubi@stones.com>
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
// ------------------------------------------------------------            
// most of this should be highly portable. hopefully ^_^
// EXCEPT for byte order assumptions. Edit sections
// marked with `intel' (grep intel *.cpp)
// I didn't use namespaces since gcc muttered something
// about namespace support being broken... whatever.

#include <string>
#include <list>
//#define LAIM_DEBUG
#define LAIM_MAJOR_VERSION 1
#define LAIM_MINOR_VERSION 1
#define LAIM_BUILD	19

enum IM_type	{ AIM, ICQ, PAL, GHOST };
// non AIM support may or may not be implemented in the future.
// GHOST is not a protocol... the last contact in the list of buddies always
// has GHOST as the associated protocol.
// I tried to make the classes "derivable"... I'm on an inheritance frenzy

enum Action	{ login, logout, mesg, error };
// after interpreting a packet from the server,
// Action IM_session::action is set to one of the above
enum Status		{ online, offline, signon, signoff, away, hidden };
enum Notice		{ NETSTAT_MESG, SERVER_MESG, CLIENT_MESG,
				ERROR_MESG, LOG_MESG, IDLE_MESG, BUDDY_LIST_MESG,
				SEARCH_RESULT_MESG};

// I'm trying to get into this "object oriented programming" thingy...
// each person on the buddylist is an object of class Contact
// they are all contained in Contacts::person which is a doubly linked list
// that is implemented by the standard library
class Contact {
protected :
	string screen_name;
	int group;
public :
	IM_type proto_id;
	string alias;
	Status status; //enum
	string mesg;	// last mesg received
	Contact::Contact();
	Contact::Contact(IM_type, string, string, Status, int);
	string send_IM(string);
	string get_profile();
	void* misc; // pointer to be used by frontend. (opt)
};

typedef list<Contact>::iterator C_LI;

class Contacts {
	// functions that monitor contact login/out
	// can update last_mod so the frontend can readily access the
	// last modified contact.
	Contact* last_mod;	// ptr to the contact who was last modified
		// modified/accessed by updated() and update()
public:
	list<Contact> person;
	// list handling routines follow:
	void add(IM_type, string, string, Status, int);
	void add(IM_type, string);
	void del(string);	// delete by alias
	C_LI find_contact_iter(string); // returns iterator for string SN
	Contact* find_contact(string);
	Contact* updated();	// returns last_mod
	void update(Contact*); // set last_mod
};

extern Contacts contact;	// in laim_contacts.cpp

// This is the "CPU" of this backend.
// my first C++ class.... this is poor design. I should
// have broken it up further.
class IM_session {
protected:
	string home_dir;	// user's home directory
	IM_type	proto_id;
	string server_ip;
	int server_port;
	bool connected;
	// sequence numbers for UDP... and FLAP
	unsigned short send_seq;
	unsigned short recv_seq;
	int socket_num;
	unsigned short status;
	unsigned short get_word();
public:
	unsigned short error_num;
	string error_str;
	Notice notice_num;
	string notice_str;
	Action action;
	string screen_name;
	string password;
	string email;
	string send_buffer;
	string recv_buffer;
	bool mesg_biff;
	void init();	
	// makes connection with server and sets up appropriate sockets
	int get_socket_num();
	void patient(bool);
};

class AIM_session : public IM_session {
private:
	string cookie;
	void BOS_login();
	void OSCAR_TLV_decode();
	string contacts_list();	// returns concatenated string composed of:
		// (1 byte for length of following SN) then SN (not NULL terminated)
		// also updates IM_session::contacts
	string profile();	// returns @olrtf (watered down html) text of user
				// profile
public:
	void login();
	void recv_buf();
	void send_buf(char);
	string TLV(unsigned short, unsigned short, string);
	string passwd_crypt(string); // de/encode string
	void SNAC_decode();
	AIM_session();
	AIM_session(string, string); // screen name, password
	~AIM_session();		// closes socket and cleans up
};

class SNAC_packet {
public:
	unsigned short genus;	// family 2bytes
	unsigned short specie;  // subtype 2bytes
	unsigned char flag[2]; // optional flags 2bytes
	long	req_id;			// request ID ?? 4 bytes
	string	data;
	SNAC_packet(unsigned short, unsigned short, char, char, long);
	SNAC_packet(string);
	void update(unsigned short, unsigned short, char, char, long);
	void decode(string);	// uses first 10 bytes of string to fill in header
							// and string[10] to end-1 for data
	string packet();			// returns SNAC packet, with proper header
};

// takes the first two bytes of s and returns its word value.
// string is automatically shortened.
unsigned short get_word(string& s);
// prepares the packet that can be sent off for a query
string search_by_email(IM_type I, string email);
	
