/*
 *  aim_login.c
 *
 *  This contains all the functions needed to actually login.
 *
 */

#include "aim.h"


/*
 * TODO: check to see if this TIS stuff still works.
 */
#ifdef TIS_TELNET_PROXY
#include "tis_telnet_proxy.h"
#endif

/*
 *  send_login(int socket, char *sn, char *password)
 *  
 * This is the initial login request packet.
 *
 * This normally contains a screen name (sn), a password,
 * and some client information.  The latter is left nearly
 * blank (it's got four zeros in it).  AIM Java puts in a 
 * bunch of junk like the client version, the amount of 
 * free memory your machine has, etc...
 *
 * The password is encoded before transmition, as per
 * encode_password().  See that function for their
 * stupid method of doing it.
 *
 *
 *
 */

int aim_send_login (struct aim_conn_t *conn, char *sn, char *password)
{
  /* this is for the client info field of this packet.  for now, just
     put a few zeros in there and hope they don't notice. */
  char info_field[] = {
    0x00, 0x00, 0x00, 0x00
  };
  int info_field_len = 4;

  char *password_encoded = NULL;  /* to store encoded password */
  int n = 0; /* counter during packet construction */

  struct command_tx_struct newpacket;

  if (conn)
    newpacket.conn = conn;
  else
    newpacket.conn = aim_getconn_type(AIM_CONN_TYPE_AUTH);

  /* breakdown of new_packet_login_len */
  newpacket.commandlen = 6; /* SNAC: fixed bytes */
  newpacket.commandlen += 2; /* SN len */
  newpacket.commandlen += strlen(sn); /* SN text */
  newpacket.commandlen += 1; /* SN null terminator */
  newpacket.commandlen += 1; /* fixed byte */
  newpacket.commandlen += 2; /* password len */
  newpacket.commandlen += strlen(password); /* password text */
  newpacket.commandlen += 1; /* password null term*/
  newpacket.commandlen += 1; /* fixed byte */
  newpacket.commandlen += 2; /* info field len */
  newpacket.commandlen += info_field_len; /* info field text */
  newpacket.commandlen += 1; /* info field null term */
  newpacket.commandlen += 41; /* fixed bytes */

  /* allocate buffer to use for constructing packet_login */
  newpacket.data = (char *) malloc ( newpacket.commandlen );
  memset(newpacket.data, 0x00, newpacket.commandlen);

  newpacket.lock = 1;
  newpacket.type = 0x01;

  /* SNAC */
  newpacket.data[0] = 0x00;
  newpacket.data[1] = 0x00;
  newpacket.data[2] = 0x00;
  newpacket.data[3] = 0x01;
  newpacket.data[4] = 0x00;
  newpacket.data[5] = 0x01;

  newpacket.data[6] = (char) ( (strlen(sn)) >> 8);
  newpacket.data[7] = (char) ( (strlen(sn)) & 0xFF);

  n = 8;
  memcpy(&(newpacket.data[n]), sn, strlen(sn));
  n += strlen(sn);
  newpacket.data[n] = 0x00;
  n++;

  newpacket.data[n] = 0x02;
  n++;

  /* store password length as word */
  newpacket.data[n] = (char) ( (strlen(password)) >> 8);
  newpacket.data[n+1] = (char) ( (strlen(password)) & 0xFF);
  n += 2;

  /* allocate buffer for encoded password */
  password_encoded = (char *) malloc(strlen(password));
  /* encode password */
  aim_encode_password(password, password_encoded);
  /* store encoded password */
  memcpy(&(newpacket.data[n]), password_encoded, strlen(password));
  n += strlen(password);
  /* free buffer */
  free(password_encoded);
  /* place null terminator after encoded password */
  newpacket.data[n] = 0x00;
  n++;

  newpacket.data[n] = 0x03;
  n++;

  newpacket.data[n] = (char) ( (info_field_len) >> 8);
  newpacket.data[n+1] = (char) ( (info_field_len) & 0xFF);
  n += 2;
  memcpy(&(newpacket.data[n]), info_field, info_field_len);
  n += info_field_len;
  newpacket.data[n] = 0x00;
  n++;

  newpacket.data[n] = 0x16;
  newpacket.data[n+1] = 0x00;
  newpacket.data[n+2] = 0x02;
  newpacket.data[n+3] = 0x00;
  n += 4;

  newpacket.data[n] = 0x01;
  newpacket.data[n+1] = 0x00;
  newpacket.data[n+2] = 0x17;
  newpacket.data[n+3] = 0x00;
  n += 4;

  newpacket.data[n] = 0x02;
  newpacket.data[n+1] = 0x00;
  newpacket.data[n+2] = 0x01;
  newpacket.data[n+3] = 0x00;
  n += 4;

  newpacket.data[n] = 0x18;
  newpacket.data[n+1] = 0x00;
  newpacket.data[n+2] = 0x02;
  newpacket.data[n+3] = 0x00;
  n += 4;

  newpacket.data[n] = 0x01;
  newpacket.data[n+1] = 0x00;
  newpacket.data[n+2] = 0x1a;
  newpacket.data[n+3] = 0x00;
  n += 4;

  newpacket.data[n] = 0x02;
  newpacket.data[n+1] = 0x00;
  newpacket.data[n+2] = 0x13;
  newpacket.data[n+3] = 0x00;
  n += 4;

  newpacket.data[n] = 0x0e;
  newpacket.data[n+1] = 0x00;
  newpacket.data[n+2] = 0x02;
  newpacket.data[n+3] = 0x75;
  n += 4;

  newpacket.data[n] = 0x73;
  newpacket.data[n+1] = 0x00;
  newpacket.data[n+2] = 0x0f;
  newpacket.data[n+3] = 0x00;
  n += 4;

  newpacket.data[n] = 0x02;
  newpacket.data[n+1] = 0x65;
  newpacket.data[n+2] = 0x6e;
  newpacket.data[n+3] = 0x00;
  n += 4;

  newpacket.data[n] = 0x09;
  newpacket.data[n+1] = 0x00;
  newpacket.data[n+2] = 0x02;
  newpacket.data[n+3] = 0x00;
  n += 4;

  newpacket.data[n] = 0x15;
  n += 1;
  
  aim_tx_enqueue(&newpacket);

  return 0;
}

/*
 *  int encode_password(
 *                      const char *password,
 *	                char *encoded
 *	                ); 
 *
 * This takes a const pointer to a (null terminated) string
 * containing the unencoded password.  It also gets passed
 * an already allocated buffer to store the encoded password.
 * This buffer should be the exact length of the password without
 * the null.  The encoded password buffer IS NOT NULL TERMINATED.
 *
 * The encoding_table seems to be a fixed set of values.  We'll
 * hope it doesn't change over time!  
 *
 */
int aim_encode_password(const char *password, char *encoded)
{
  char encoding_table[] = {
    0xf3, 0xb3, 0x6c, 0x99,
    0x95, 0x3f, 0xac, 0xb6,
    0xc5, 0xfa, 0x6b, 0x63,
    0x69, 0x6c, 0xc3, 0x9f
  };

  int i;
  
  for (i = 0; i < strlen(password); i++)
      encoded[i] = (password[i] ^ encoding_table[i]);

  return 0;
}




