#include "aimt.h"

handle_status(conn * c, xpt * x)
{
  jpair *to, *jnext;
  char *from_user, *from_nick, *say, *ext, *icon, *pristr, *statstr;
  int status, priority;
  xpt *next, *next2;

  DBG_MSG("handle_status");

  if (c == NULL || x == NULL)
    return;

  from_user = jpair_getvalnew(x->attributes, "from");
  from_nick = jpair_getvalnew(x->attributes, "nick");
  to = NULL;
  if (from_user == NULL)
  {
    DBG_MSG("Invalid status, improper from");
    return;
  }

  status = JSTATUS_NORMAL;
  statstr = jpair_getval(x->attributes, "type");
  if (statstr != NULL && strcmp(statstr, "online") == 0)
    status = JSTATUS_ONLINE;
  if (statstr != NULL && strcmp(statstr, "offline") == 0)
    status = JSTATUS_OFFLINE;

  pristr = icon = say = ext = NULL;
  priority = 0;

  next = x->children;
  while (next != NULL)
  {
    if (next->type == XPT_TAG && strcmp(next->name, "to") == 0)
      to = jpair_add(to, xpt2str_chain(next->children), jpair_getvalnew(next->attributes, "nick"));
    if (next->type == XPT_TAG && strcmp(next->name, "say") == 0)
      say = xpt2str_chain(next->children);
    if (next->type == XPT_TAG && strcmp(next->name, "icon") == 0)
      icon = xpt2str_chain(next->children);
    if (next->type == XPT_TAG && strcmp(next->name, "ext") == 0)
    {
      next2 = next->children;
      ext = NULL;
      while (next2 != NULL)
      {
	ext = strgrow(ext, xpt2str(next2), 1, 1);
	next2 = next2->next;
      }
    }
    if (next->type == XPT_TAG && strcmp(next->name, "priority") == 0)
    {
      pristr = xpt2str_chain(next->children);
      sscanf(pristr, "%d", &priority);
    }
    next = next->next;
  }

  jnext = to;
  while (jnext != NULL)
  {
    status_update(from_user, from_nick, jnext->key, status, say, icon, ext, priority);
    jnext = jnext->next;
  }

  free(from_user);
  free(from_nick);
  free_jpair(to);
  free(icon);
  free(say);
  free(ext);
  free(pristr);
}


void status_update(char *user, char *nick, char *to, int status, char *say, char *icon, char *ext, int priority)
{
  xpt *parent, *x;
  char intbuff[20], *server;
  int to_uin;

  parent = xpt_new_tag(NULL, "status", NULL);

  switch (status)
  {
  case JSTATUS_NORMAL:
    break;
  case JSTATUS_ONLINE:
    parent->attributes = jpair_new(NULL, "type", "online", 0);
    break;
  case JSTATUS_OFFLINE:
    parent->attributes = jpair_new(NULL, "type", "offline", 0);
    break;
  }

  if (say != NULL)
  {
    x = xpt_new_tag(parent, "say", NULL);
    xpt_new_cdata(x, say);
  }

  if (icon != NULL)
  {
    x = xpt_new_tag(parent, "icon", NULL);
    xpt_new_cdata(x, icon);
  }

  if (ext != NULL)
  {
    x = xpt_new_tag(parent, "ext", NULL);
    xpt_new_cdata(x, ext);
    x->children->name = strdup("NOESCAPE");	/* kludge so that it isn't escaped automatically */
  }

  if (priority != 0)
  {
    x = xpt_new_tag(parent, "priority", NULL);
    sprintf(intbuff, "%d", priority);
    xpt_new_cdata(x, intbuff);
  }


  DBG_VAL("status", xpt2str(parent));

  status_manager(status, nick, say, icon, priority);

  server = strstr(to, "@");
  if(server != NULL)
    server = '\0';
  sscanf(to,"%d",&to_uin);
  AddContact(to_uin);

  free_xpt(parent);
}


void status_manager(int type, char *nick, char *say, char *icon, int priority)
{
  int new = 0;
  jpair *sess, *pri;

  sess = jpair_get(etc.sessions, nick);
  if(sess == NULL)
  {
    new = 1;
    etc.sessions = jpair_new(etc.sessions, nick, say, priority);
    sess = jpair_get(etc.sessions, nick);
  }else{
    sess->flag = priority;
  }

  if(new == 1 && sess == get_primary() && type != JSTATUS_OFFLINE)
  {  /* new session is now the prmary one, get online if we're not already */
    etc.status = JSTATUS_ONLINE;
    ICQ_Change_Status(STATUS_ONLINE);
    return;
  }

  if(type == JSTATUS_OFFLINE)
  {
    etc.sessions = jpair_del(etc.sessions, nick);
    sess = get_primary();
    if(sess == NULL)
    {
      etc.status = JSTATUS_OFFLINE;
      ICQ_Change_Status(STATUS_OFFLINE);
      return;
    }
  }

  if(sess == get_primary())
  {
    /* from here we modify the ICQ status fuzzily to the incoming status change for the primary session */
  }
}


jpair *get_primary()
{
  jpair *next, *primary;

  primary = NULL;
  next = etc.sessions;
  while(next != NULL)
  {
    if(primary == NULL)
      if(next->flag >= 0)
        primary = next;
    if(next->flag >= 0 && next->flag < primary->flag)
      primary = next;
    next = next->next;
  }

  return primary;
}

