#include "aimt.h"

void sig_die()
{
  ERROR("main", "Exiting Template Transport");
  longjmp(_jabberlib_ioenv, 1);
}

char *make_tag_quick(char *name, char *value)
{
  char *ret;

  if (value == NULL)
    return NULL;

  if (*value == '\0')
    return NULL;

  value = strescape(value);
  ret = malloc(strlen(name) * 2 + strlen(value) + 6);
  sprintf(ret, "<%s>%s</%s>", name, value, name);
  free(value);

  return ret;
}

void logger(int level, char *from, char *message)
{
  static time_t t;
  static int log, checklog;
  char *tstr, *tmpctime;
  int check;

  /* initialize and open the logfile if configured to do so */
  if (checklog != 1)
  {
    log = -1;
    if (jpair_get(etc.vars, "errorlog") != NULL)
    {
      checklog = 1;
      log = open(jpair_getval(etc.vars, "errorlog"), O_WRONLY | O_CREAT | O_APPEND);
      if (log < 0)
	ERROR("main", "Unable to open errorlog");
    }
  }

  /* log only at the level told to do so */
  if (level >= etc.debug)
  {
    time(&t);
    tstr = strgrow(NULL, "[", 0, 0);
    tmpctime = strdup(ctime(&t));
    tmpctime[strlen(tmpctime) - 1] = '\0';
    tstr = strgrow(tstr, tmpctime, 1, 1);
    tstr = strgrow(tstr, "] ", 1, 0);
    tstr = strgrow(tstr, from, 1, 0);
    tstr = strgrow(tstr, " \"", 1, 0);
    if (message != NULL)
      tstr = strgrow(tstr, message, 1, 0);
    tstr = strgrow(tstr, "\"\n", 1, 0);

    /* if a logfile is configured and opened, put logs there unless were debugging */
    if (log >= 0 && etc.debug > SYSLOG_DEBUG)
    {
      check = write(log, tstr, strlen(tstr));
      if (check != strlen(tstr))
      {
	log = -1;
	ERROR("main", "Writing to error log file failed!");
	syslog(level, from, message);
      }
    }
    else
    {
      printf("%s", tstr);
    }

    free(tstr);
  }
}

void transport_deliver(char *from_user, char *from_nick, char *to_user, char *to_nick, xpt * x)
{
  xpt *to;
  char *str, *packet, *server;
  jpair *attributes = NULL;

  server = strstr(to_user, "@");
  if (server == NULL)
    return;
  else
    ++server;

  from_user = strgrow(NULL, from_user, 0, 0);
  from_user = strgrow(from_user, "@", 1, 0);
  from_user = strgrow(from_user, jpair_getval(etc.vars, "name"), 1, 0);
  x->attributes = jpair_new(x->attributes, "from", from_user, 0);
  x->attributes = jpair_new(x->attributes, "nick", from_nick, 0);
  free(from_user);

  to = xpt_new_tag(x, "to", NULL);
  if (to_nick != NULL)
    to->attributes = jpair_new(to->attributes, "nick", to_nick, 0);
  xpt_new_cdata(to, to_user);

  attributes = jpair_new(attributes, "to", server, 0);
  attributes = jpair_new(attributes, "from", jpair_getval(etc.vars, "name"), 0);
  str = xpt2str(x);
  packet = create_packet("r", attributes, str);

  etc.transport->packets = packet_add(etc.transport->packets, packet, from_user);

  free_jpair(attributes);
  free(str);
  free(packet);
}

char *create_packet(char *name, jpair * atts, char *body)
{
  xpt *next;
  char *ret = NULL;
  char *buf = NULL;

  ret = strgrow(ret, "<", 1, 0);
  ret = strgrow(ret, name, 1, 0);
  while (atts != NULL)
  {
    ret = strgrow(ret, " ", 1, 0);
    ret = strgrow(ret, atts->key, 1, 0);
    ret = strgrow(ret, "='", 1, 0);
    ret = strgrow(ret, strescape(atts->val), 1, 1);
    ret = strgrow(ret, "'", 1, 0);
    atts = atts->next;
  }
  ret = strgrow(ret, "><![CDATA[", 1, 0);
  ret = strgrow(ret, body, 1, 0);
  ret = strgrow(ret, "]]></", 1, 0);
  ret = strgrow(ret, name, 1, 0);
  ret = strgrow(ret, ">", 1, 0);

  return ret;
}

