#include "jabber.h"

/* info packets <info></info>

 * Have:
 *      <to></to> containing user account/id, defaults to sender
 *      <target></target> containing the target/zone for the info
 *      <type></type> containing the area within the target for the info
 * Then, query can be either:
 *      <query></query> for fetching results
 *      <result></result> for the response
 *      <set></set> to save new values
 */

void handle_info(conn * c, xpt * x)
{
  jpair *from, *to, *query;
  xpt *next;
  handler *h;
  int target, query_type;
  char *type, *str;

  DBG_MSG("handle_info");

  if (c == NULL || x == NULL)
    return;

  from = get_from(c, x);
  if (from == NULL)
  {
    DBG_MSG("Invalid query, unknown or illegal sender");
    return;
  }

  to = get_to(c, x);
  if (to == NULL)		/* default to self if no <to></to> specified */
  {
    to = jpair_dup(from);
  }

/*
   h = modules_gethandler_user(API_INFO, from->key);
   if(h == NULL || h->h_info == NULL)
   {
   DBG_MSG("Invalid query, no handler defined");
   return;
   }
 */

  query = NULL;
  query_type = INFO_NONE;
  target = INFO_NONE;
  next = x->children;
  while (next != NULL)
  {
    if (next->type == XPT_TAG && strcmp(next->name, "type") == 0)
      type = xpt2str_chain(next->children);
    if (next->type == XPT_TAG && strcmp(next->name, "target") == 0)
    {
      str = xpt2str_chain(next->children);
      if (streq(str, "server"))
	target = INFO_2SERVER;
      if (streq(str, "user"))
	target = INFO_2USER;
      if (streq(str, "client"))
	target = INFO_2CLIENT;
      free(str);
    }
    if (next->type == XPT_TAG && strcmp(next->name, "query") == 0)
      query_type = INFO_QUERY;
    if (next->type == XPT_TAG && strcmp(next->name, "result") == 0)
      query_type = INFO_RESULT;
    if (next->type == XPT_TAG && strcmp(next->name, "set") == 0)
      query_type = INFO_SET;
    if (query != NULL && query_type != INFO_NONE)
    {
      query = xpt2jpair(next->children);
    }
    next = next->next;
  }

  /* !!!! need to do delivery/processing logic now */
  /* anything of target=client send on immediately */
  /* anything of target=server process internally */
  /* anything of target=user send to module */

  free_jpair(to);
  free_jpair(from);
}

void info_send(char *from_user, char *from_nick, char *to_user, char *to_nick, int target, char *type, jpair *query, int query_type)
{ 
	xpt *parent, *x;
	char intbuff[20], *server;
	session *s;

	parent = xpt_new_tag(NULL, "info", NULL);

	x = xpt_new_tag(parent, "target", NULL);
	switch(target)
	{
		case INFO_2CLIENT:
                        xpt_new_cdata(x, "client");
			break;
		case INFO_2SERVER:
                        xpt_new_cdata(x, "server");
			break;
		case INFO_2USER:
                        xpt_new_cdata(x, "user");
			break;
	}

	x = xpt_new_tag(parent, "type", NULL);
	xpt_new_cdata(x, type);

	switch(query_type)
	{
		case INFO_QUERY:
                	x = xpt_new_tag(parent, "query", NULL);
			break;
		case INFO_RESULT:
                	x = xpt_new_tag(parent, "result", NULL);
			break;
		case INFO_SET:
                	x = xpt_new_tag(parent, "set", NULL);
			break;
	}


	server = strstr(to_user, "@");
	if(server != NULL)
	{
		++server;

		if(islocal(server) == FALSE)
		{
			DBG_VAL("info_to_server",server);
			transport_deliver(from_user, from_nick, to_user, to_nick, parent);
			return;
		}else{
			/* fix the userid */
			--server;
			*server = '\0';
		}
	}


	/* only local client see the <from></from> tag */
	x = xpt_new_tag(parent, "from", NULL);
	if(from_nick != NULL)
		x->attributes = jpair_new(x->attributes, "nick", from_nick, 0);
	xpt_new_cdata(x, from_user);

	/* find the local user this is to and send away */
	s = session_lookup(NULL, to_user, NULL);
	while(s != NULL)
	{
		if(streq(to_user, s->user) && streq(to_nick, s->nick))
			local_deliver(from_user, s->user, s->nick, s->c, parent);
		s = s->next;
	}

	free_xpt(parent);
}


