/* 
	Jabber mysql module.

	Nicholas Kirsch
	nick@kirsch.org
*/

#include <mysql/mysql.h>
#include <string.h>
#include "jabber.h"

module mod_mysql;

struct mysql_list
{
	char *id;
	struct mysql_list *next;
};

struct mysql_group
{
	char *id;
	char *group;
	struct mysql_list *members;
	struct mysql_group *next;
};

struct mysql_group *mysql_myusers = NULL;
struct mysql_list *mysql_otherusers = NULL;
jpair *mysql_authusers = NULL;

MYSQL * mysql_connection = NULL;

struct mysql_data_set
{
	char * user;
	char * pass;
	char * host;
	char * database;
};

struct mysql_data_set * mysql_data;

int mysql_getlist(char *id, struct mysql_list *list)
{
	DBUG("MOD_MYSQL: getlist for id ",id)

	if(id == NULL || list == NULL)
		return 0;
	while(list != NULL)
	{
		if(strcmp(id, list->id) == 0)
			return 1;
		list = list->next;
	}
	return 0;
}

struct mysql_list *mysql_setlist(char *id, struct mysql_list *list)
{
	struct mysql_list *new;

	DBUG("MOD_MYSQL: setlist for id ",id)

	if(id == NULL)
		return NULL;
	if(mysql_getlist(id,list) == 0)
	{
		new = malloc(sizeof(struct mysql_list));
		new->id = strdup(id);
		new->next = list;
		return new;
	}
	return list;
}

struct mysql_list *mysql_freelist(char *id, struct mysql_list *list)
{
	struct mysql_list *prev, *next;
	int special = 0;

	DBUG("MOD_MYSQL: freelist for id ",id)

	prev = next = list;

	if(list != NULL && list->next == NULL)
		special = 1;

	if(id == NULL)
		return NULL;
	while(next != NULL)
	{
		if(strcmp(id, next->id) == 0)
		{
			prev->next = next->next;
			free(next->id);
			free(next);
			if(special)
				return NULL;
			return list;
		}
		prev = next;
		next = next->next;
	}
	return list;
}

void mysql_groupadd(char *id, char *group, char *newid)
{
	struct mysql_group *new, *all = mysql_myusers;

	DBUG("MOD_MYSQL: groupadd for id ",id)

	if(id == NULL || group == NULL || newid == NULL)
		return;

	while(all != NULL)
	{
		if(strcmp(id, all->id) == 0 && strcmp(group, all->group) == 0)
		{
			all->members = mysql_setlist(newid,all->members);
			return;
		}
		all = all->next;
	}
	new = malloc(sizeof(struct mysql_group));
	new->id = strdup(id);
	new->group = strdup(group);
	new->members = mysql_setlist(newid,NULL);
	new->next = mysql_myusers;
	mysql_myusers = new;
}

void mysql_groupdel(char *id, char *group, char *newid)
{
	struct mysql_group *all = mysql_myusers;

	DBUG("MOD_MYSQL: groupdel for id ",id)

	if(id == NULL || group == NULL || newid == NULL)
		return;

	while(all != NULL)
	{
		if(strcmp(id, all->id) == 0 && strcmp(group, all->group) == 0)
		{
			all->members = mysql_freelist(newid,all->members);
			return;
		}
		all = all->next;
	}
}

jpair *mysql_jpairgroup(char *id, char *group)
{
	struct mysql_group *all = mysql_myusers;
	struct mysql_list *next;
	jpair *new = NULL, *par = NULL;

	DBUG("MOD_MYSQL: create jpair from group for id ",id)

	if(id == NULL)
		return NULL;

	if(group == NULL)
	{
		while(all != NULL)
		{
			if(strcmp(id, all->id) == 0)
			{
				new = malloc(sizeof(jpair));
				new->key = NULL;
				new->val = strdup(all->group);
				new->next = par;
				par = new;
			}
			all = all->next;
		}
		return par;
	}

	while(all != NULL)
	{
		if(strcmp(id, all->id) == 0 && strcmp(group, all->group) == 0)
		{
			next = all->members;
			while(next != NULL)
			{
				new = malloc(sizeof(jpair));
				new->val = strdup(group);
				new->key = strdup(next->id);
				new->next = par;
				par = new;
				next = next->next;
			}
			return par;
		}
		all = all->next;
	}
	return NULL;
}

/*
typedef struct
{
  char *name;
  jpair *attributes;
  char *contents;
} tag;
*/ 

MYSQL * mysql_init_connection()
{
		if (mysql_data == 0)
		{
			DBUG ("MOD_MYSQL : MYSQL_DATA EMPTY!", "")
			return (MYSQL *)0;
		}
		mysql_connection = mysql_init (0);
		return (mysql_connection = mysql_real_connect (mysql_connection, mysql_data->host, mysql_data->user, mysql_data->pass, mysql_data->database, 0, 0, 0));
}
	

void mysql_handle_config(tag *t, void *null)
{

	/* Apparently the strings from the tag are not destroyed. I would rather do a strdup here, instead of using the
		initial string, but this is faster. */

	if (mysql_data == 0)
	{
		mysql_data = (struct mysql_data_set *)malloc (sizeof(struct mysql_data_set));
	}

	if (strcmp (t->name, "user") == 0)
	{
		mysql_data->user = strdup(t->contents);
		DBUG("MOD_MYSQL: handler_config : user", mysql_data->user)
	}
	if (strcmp (t->name, "host") == 0)
	{
		mysql_data->host = strdup(t->contents);
		DBUG("MOD_MYSQL: handler_config : host", mysql_data->host)
	}
	if (strcmp (t->name, "pass") == 0)
	{
		mysql_data->pass = strdup(t->contents);
		DBUG("MOD_MYSQL: handler_config : pass", mysql_data->pass)
	}
	if (strcmp (t->name, "db") == 0)
	{
		mysql_data->database = strdup(t->contents);
		DBUG("MOD_MYSQL: handler_config : db", mysql_data->database)
	}
}

/* I assume the authenicate function should perform a lookup in the mysql database. It will need to return
	a positive number (2) upon success, and -1 upon error. 
	I will need to build mysql query.. SELECT * FROM users WHERE username = 'username' AND password = 'password'
 */

int mysql_authenticate(char *id, char *pass)
{
	MYSQL_RES * mysql_result;
	char * query_string;
	
	DBUG("MOD_MYSQL: handler_auth ",id)

	if (mysql_connection == 0)
	{
		if (mysql_init_connection() == 0)
		{
			DBUG ("MOD_MYSQL : DATABASE CONNECTION FAILED!", "")
			return 2;
		}
	}

	query_string = (char *) malloc(60 + strlen(id) + strlen(pass));
	sprintf(query_string, "SELECT * FROM users WHERE username = '%s' AND password = '%s'", id, pass);	
	if (mysql_query (mysql_connection, query_string) != 0)
	{
		DBUG("MOD_MYSQL: QUERY FAILED!", "")
		/* TODO : For now, accept user if query fails. Ideally it should only fail in the event of a database mis-configuration
							or failure.
		*/
		return 2;
	}
	if (mysql_num_rows(mysql_result) == 0)
	{
		DBUG("MOD_MYSQL: AUTH FAILED!", "")
		return -1;
	}
	return 2;
}

int mysql_notify(char *id, int status)
{
	/* if offline freelist, if online setlist */
	DBUG("MOD_MYSQL: handler_notify ",id)
	return 1;
}

jpair *mysql_status(char *id, int status)
{
	DBUG("MOD_MYSQL: handler_status ",id)
	return mysql_jpairgroup(id,"_all");
}

jpair *mysql_roster(int type, char *id, char *group, char *newid)
{
	DBUG("MOD_MYSQL: handler_roster ",id)
	switch(type)
	{
		case ROSTER_REMOVE:
			mysql_groupdel(id, group, newid);
			mysql_groupdel(id, "_all", newid);
			break;
		case ROSTER_ADD:
			mysql_groupadd(id, group, newid);
			mysql_groupadd(id, "_all", newid);
			break;
		case ROSTER_GET:
			return mysql_jpairgroup(id,group);
			break;
	}
	return NULL;
}

jpair *mysql_message_put(char *to, jpair *p)
{
	DBUG("MOD_MYSQL: handler_message_put ",to)
	return NULL;
}

jpair *mysql_message_get(char *to)
{
	DBUG("MOD_MYSQL: handler_message_get ",to)
	return NULL;
}

/** TO UNDERSTAND
typedef struct
{
  void (* h_char)();
  void (* h_tag)();
  int whitespace; 
} tap; 


**/
void mysql_handler_init(jpair *p)
{
	char *config;
	static tap tp = {NULL,&mysql_handle_config,0};

	config = jpair_getval(p, "mysql");
	DBUG("MOD_MYSQL: handler_init ",config)

	tag_parse(&tp,config,NULL);


}

int mysql_handle(package *p)
{
	DBUG("MOD_MYSQL: handler_handle package for ",p->id)
	return 0;
}

void mysql_log (char * message)
{
	if ( (mysql_connection == 0) && (mysql_init_connection() == 0))
		return;
	mysql_query (mysql_connection, "INSERT INTO log (message) VALUES ('%s')");
}
	

module mod_mysql =
{
	0,
	mysql_handler_init,	/* initializes handler */
	mysql_authenticate,	/* authenticate user */
	mysql_notify,				/* notification ? */
	mysql_status,				/* change status ? */
	mysql_roster,				/* add to roster ? */
	mysql_message_put,	/* offline_message */
	mysql_message_get,	/* online_message */
	NULL,								/* search */
	mysql_log								/* log */
};
