#include "template.h"

conn *dispatch(char *p, conn *c, int flag)
{
	conn *c_new = NULL;
	xpt *xml, *next;
	int sock;

	DBG_VAL("dispatch_data",p);

	switch(flag)
	{
		case IO_REMOVE:
			DBG_VAL("removing_connection",c->name)
			if(c == etc.transport)
			{
				ERROR("etherx", "Transport connection failure, exiting");
				/* should start special loop to re-connect to transport with timeout, if reconnect, reconfigure, etc */
				sig_die();
			}else{
				(int)c_new = -1;
			}
			break;

		case IO_INIT:
			DBG_MSG("Initialization from IO loop");

			/* connect to the local etherx */
			sock = make_netsocket(DEFAULT_ETHERXPORT, jpair_getval(etc.vars, "local"), 1);
			if(sock < 0)
			{
				ERROR("etherx", "Unable to connect to Transport, exiting");
				sig_die();
			}
			c_new = conn_add(NULL, sock);
			c_new->packets = packet_add(c_new->packets,"<?xml version='1.0'?><ether><handshake>",NULL);
			c_new->packets = packet_add(c_new->packets,DEFAULT_ID,NULL);
			c_new->packets = packet_add(c_new->packets,"</handshake>",NULL);
			etc.transport = c_new; /* store globally */
			break;

		case IO_NORMAL:
			DBG_MSG("Processing Normally");
			if(c->xp == NULL) /* initialize xp if it hasn't been yet */
				c->xp = xpt_packet_new();
			xpt_packet_data(c->xp,p,strlen(p));

			next = xml = c->xp->packets;
			while(next != NULL)
			{
				xml = next;
				next = next->next;
				xml->next = NULL; /* single it out */
				handle_packet(c, xml);
			}
			free_xpt(c->xp->packets);
			c->xp->packets = NULL;
			if(c->xp->flag != XPT_POOLFLAG_NONE)
			{
				switch(c->xp->flag)
				{
					case XPT_POOLFLAG_ERROR:
						syslog_debug(SYSLOG_ERROR, "connection", "INVALID_XML_FROM", c->name);
						break;
					case XPT_POOLFLAG_CLOSE:
						DBG_VAL("connection_xml_close",c->name);
						break;
					default:
						DBG_VAL("unknown_xml_error",c->name);
				}
				/* !!! should send error message back about this */
				(int)c_new = -1;
			}
			break;
	}

	return c_new;
}

void handle_packet(conn *c, xpt *x)
{
	xpt *xml, *next;
	char *val = NULL;
	char *type;

	DBG_VAL("handling_normal_packet",c->name);

	type = x->name;
	if(type == NULL) /* not a valid incoming packet */
	{
		return;
	}

	/* stringpieces->string->xpt to make sure we get xpt in the end */
	next = x->children;
	while(next != NULL)
	{
		if(next->type == XPT_CDATA)
			val = strgrow(val,next->value,1,0);
		next = next->next;
	}
	DBG_VAL("transport_packet",val);

	xml = xpt_str(val);
	free(val);

	if(NULL == xml)
	{
		ERROR("template","Got some bad XML!");
		return;
	}

	if(strcmp(x->name,"r") == 0)
	{
		DBG_MSG("Handling remote packet from transport");
		if(strcmp(xml->name,"message") == 0)
			handle_message(c, xml);
		if(strcmp(xml->name,"status") == 0)
			handle_status(c, xml);
		free_xpt(xml);
		return;
	}

	if(strcmp(x->name,"error") == 0)
	{
		DBG_MSG("Handling bounced packet from transport");
		free_xpt(xml);
		return;
	}

	if(strcmp(x->name,"config") == 0)
	{
		DBG_MSG("Handling config packet from transport");
		config_handle(xml->children);
		/* don't free the config, is referenced throughout apps lifetime */
		return;
	}

	syslog_debug(SYSLOG_ERROR, "etherx", "unknown_packet", x->name);	
}


void transport_deliver(char *from_user, char *from_nick, char *to_user, char *to_nick, xpt *x)
{
	xpt *to;
	char *str, *packet, *server;
	jpair *attributes = NULL;

	server = strstr(to_user, "@");
	if(server == NULL)
		return;
	else
		++server;

	from_user = strgrow(NULL,from_user,0,0);
	from_user = strgrow(from_user,"@",1,0);
	from_user = strgrow(from_user,jpair_getval(etc.vars,"name"),1,0);
	x->attributes = jpair_new(x->attributes, "from", from_user, 0);
	x->attributes = jpair_new(x->attributes, "nick", from_nick, 0);
        free(from_user);

	to = xpt_new_tag(x, "to", NULL);
	if(to_nick != NULL)
		to->attributes = jpair_new(to->attributes, "nick", to_nick, 0);
	xpt_new_cdata(to, to_user);

	attributes = jpair_new(attributes, "to", server, 0);
	attributes = jpair_new(attributes, "from", jpair_getval(etc.vars, "name"), 0);
	str = xpt2str(x);
	packet = create_packet("r", attributes, str);

	etc.transport->packets = packet_add(etc.transport->packets, packet, from_user);

	free_jpair(attributes);
	free(str);
	free(packet);
}

char *create_packet(char *name, jpair *atts, char *body)
{
	xpt *next;
	char *ret = NULL;
	char *buf = NULL;

	ret = strgrow(ret,"<",1,0);
	ret = strgrow(ret,name,1,0);
	while(atts != NULL)
	{
		ret = strgrow(ret," ",1,0);
		ret = strgrow(ret,atts->key,1,0);
		ret = strgrow(ret,"='",1,0);
		ret = strgrow(ret,strescape(atts->val),1,1);
		ret = strgrow(ret,"'",1,0);
		atts = atts->next;
	}
	ret = strgrow(ret,"><![CDATA[",1,0);
	ret = strgrow(ret,body,1,0);
	ret = strgrow(ret,"]]></",1,0);
	ret = strgrow(ret,name,1,0);
	ret = strgrow(ret,">",1,0);

	return ret;
}

