#include "template.h"

handle_status(conn *c, xpt *x)
{
	jpair *to, *jnext;
	char *from_user, *from_nick, *say, *ext, *icon, *pristr, *statstr;
	int status, priority;
	xpt *next, *next2;

	DBG_MSG("handle_status");

	if(c == NULL || x == NULL)
		return;

	from_user = jpair_getvalnew(x->attributes, "from");
	from_nick = jpair_getvalnew(x->attributes, "nick");
	to = NULL;
	if(from_user == NULL)
	{
		DBG_MSG("Invalid status, improper from");
		return;
	}

	status = STATUS_NORMAL;
	statstr = jpair_getval(x->attributes, "type");
	if(statstr != NULL && strcmp(statstr, "online") == 0)
		status = STATUS_ONLINE;
	if(statstr != NULL && strcmp(statstr, "offline") == 0)
		status = STATUS_OFFLINE;

	pristr = icon = say = ext = NULL;
	priority = 0;

	next = x->children;
	while(next != NULL)
	{
		if(next->type == XPT_TAG && strcmp(next->name, "to") == 0)
			to = jpair_add(to,xpt2str_chain(next->children),jpair_getvalnew(next->attributes, "nick"));
		if(next->type == XPT_TAG && strcmp(next->name, "say") == 0)
			say = xpt2str_chain(next->children);
		if(next->type == XPT_TAG && strcmp(next->name, "icon") == 0)
			icon = xpt2str_chain(next->children);
		if(next->type == XPT_TAG && strcmp(next->name, "ext") == 0)
		{
			next2 = next->children;
			ext = NULL;
			while(next2 != NULL)
			{
				ext = strgrow(ext, xpt2str(next2), 1, 1);
				next2 = next2->next;
			}
		}
		if(next->type == XPT_TAG && strcmp(next->name, "priority") == 0)
		{
			pristr = xpt2str_chain(next->children);
			sscanf(pristr,"%d",&priority);
		}
		next = next->next;
	}

	jnext = to;
	while(jnext != NULL)
	{
		status_update(from_user, from_nick, jnext->key, status, say, icon, ext, priority); 
		jnext = jnext->next;
	}

	free(from_user);
	free(from_nick);
	free_jpair(to);
	free(icon);
	free(say);
	free(ext);
	free(pristr);
}


void status_update(char *user, char *nick, char *to, int status, char *say, char *icon, char *ext, int priority)
{ 
	xpt *parent, *x;
	char intbuff[20], *server;

	parent = xpt_new_tag(NULL, "status", NULL);

	switch(status)
	{
		case STATUS_NORMAL:
			break;
		case STATUS_ONLINE:
			parent->attributes = jpair_new(NULL, "type", "online", 0);
			break;
		case STATUS_OFFLINE:
			parent->attributes = jpair_new(NULL, "type", "offline", 0);
			break;
	}

	if(say != NULL)
	{
		x = xpt_new_tag(parent, "say", NULL);
		xpt_new_cdata(x, say);
	}

	if(icon != NULL)
	{
		x = xpt_new_tag(parent, "icon", NULL);
		xpt_new_cdata(x, icon);
	}

	if(ext != NULL)
	{
		x = xpt_new_tag(parent, "ext", NULL);
		xpt_new_cdata(x, ext);
		x->children->name = strdup("NOESCAPE"); /* kludge so that it isn't escaped automatically */
	}

	if(priority != 0)
	{
		x = xpt_new_tag(parent, "priority", NULL);
		sprintf(intbuff,"%d",priority);
		xpt_new_cdata(x, intbuff);
	}


	DBG_VAL("status",xpt2str(parent));

	/* handle status here */

	free_xpt(parent);
}


